'''OpenGL extension APPLE.vertex_array_object

Overview (from the spec)
	
	This extension introduces named vertex array objects which encapsulate
	vertex array state on the client side. The main purpose of these 
	objects is to keep pointers to static vertex data and provide a name 
	for different sets of static vertex data.  
	
	By extending vertex array range functionality this extension allows multiple
	vertex array ranges to exist at one time, including their complete sets of
	state, in manner analogous to texture objects. 
	
	GenVertexArraysAPPLE creates a list of n number of vertex array object
	names.  After creating a name, BindVertexArrayAPPLE associates the name with
	a vertex array object and selects this vertex array and it's associated
	state as current.  To get back to the default vertex array and its
	associated state the client should bind to vertex array named 0.
	
	Once a client is done using a vertex array object it can be deleted with
	DeleteVertexArraysAPPLE.  The client is responsible for allocating and
	deallocating the memory used by the vertex array data, while the
	DeleteVertexArraysAPPLE command deletes vertex array object names and
	associated state only.

The official definition of this extension is available here:
	http://oss.sgi.com/projects/ogl-sample/registry/APPLE/vertex_array_object.txt

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_APPLE_vertex_array_object'
GL_VERTEX_ARRAY_BINDING_APPLE = constant.Constant( 'GL_VERTEX_ARRAY_BINDING_APPLE', 0x85B5 )
glBindVertexArrayAPPLE = platform.createExtensionFunction( 
	'glBindVertexArrayAPPLE', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLuint,),
	doc = 'glBindVertexArrayAPPLE( GLuint(array) ) -> None',
	argNames = ('array',),
)

glDeleteVertexArraysAPPLE = platform.createExtensionFunction( 
	'glDeleteVertexArraysAPPLE', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLsizei, arrays.GLuintArray,),
	doc = 'glDeleteVertexArraysAPPLE( GLsizei(n), GLuintArray(arrays) ) -> None',
	argNames = ('n', 'arrays',),
)

glGenVertexArraysAPPLE = platform.createExtensionFunction( 
	'glGenVertexArraysAPPLE', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLsizei, arrays.GLuintArray,),
	doc = 'glGenVertexArraysAPPLE( GLsizei(n), GLuintArray(arrays) ) -> None',
	argNames = ('n', 'arrays',),
)

glIsVertexArrayAPPLE = platform.createExtensionFunction( 
	'glIsVertexArrayAPPLE', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=constants.GLboolean, 
	argTypes=(constants.GLuint,),
	doc = 'glIsVertexArrayAPPLE( GLuint(array) ) -> constants.GLboolean',
	argNames = ('array',),
)


def glInitVertexArrayObjectAPPLE():
	'''Return boolean indicating whether this extension is available'''
	return extensions.hasGLExtension( EXTENSION_NAME )



"""Implementation of GLU Nurbs structure and callback methods

Same basic pattern as seen with the gluTess* functions, just need to
add some bookkeeping to the structure class so that we can keep the
Python function references alive during the calling process.
"""
from OpenGL.raw import GLU as simple
from OpenGL import platform, converters, wrapper
from OpenGL.GLU import glustruct
from OpenGL.lazywrapper import lazy
from OpenGL import arrays
import ctypes
import weakref
from OpenGL.platform import PLATFORM

__all__ = (
	'GLUnurbs',
	'gluNewNurbsRenderer',
	'gluNurbsCallback',
	'gluNurbsCallbackData',
	'gluNurbsCallbackDataEXT',
	'gluNurbsCurve',
	'gluNurbsSurface',
	'gluPwlCurve',
)

# /usr/include/GL/glu.h 242
class GLUnurbs(glustruct.GLUStruct, simple.GLUnurbs):
	"""GLU Nurbs structure with oor and callback storage support
	
	IMPORTANT NOTE: the texture coordinate callback receives a raw ctypes 
	data-pointer, as without knowing what type of evaluation is being done 
	(1D or 2D) we cannot safely determine the size of the array to convert 
	it.  This is a limitation of the C implementation.  To convert to regular 
	data-pointer, just call yourNurb.ptrAsArray( ptr, size, arrays.GLfloatArray )
	with the size of data you expect.
	"""
	FUNCTION_TYPE = PLATFORM.functionTypeFor(PLATFORM.GLU)
	CALLBACK_FUNCTION_REGISTRARS = {
		# mapping from "which" to a function that should take 3 parameters,
		# the nurb, the which and the function pointer...
	}
	CALLBACK_TYPES = {
		# mapping from "which" GLU enumeration to a ctypes function type
		simple.GLU_NURBS_BEGIN: FUNCTION_TYPE( 
			None, simple.GLenum 
		),
		simple.GLU_NURBS_BEGIN_DATA: FUNCTION_TYPE( 
			None, simple.GLenum, ctypes.POINTER(simple.GLvoid) 
		),
		simple.GLU_NURBS_VERTEX: FUNCTION_TYPE( 
			None, ctypes.POINTER(simple.GLfloat)
		),
		simple.GLU_NURBS_VERTEX_DATA: FUNCTION_TYPE( 
			None, ctypes.POINTER(simple.GLfloat), ctypes.POINTER(simple.GLvoid) 
		),
		simple.GLU_NURBS_NORMAL: FUNCTION_TYPE( 
			None, ctypes.POINTER(simple.GLfloat)
		),
		simple.GLU_NURBS_NORMAL_DATA: FUNCTION_TYPE( 
			None, ctypes.POINTER(simple.GLfloat), ctypes.POINTER(simple.GLvoid) 
		),
		simple.GLU_NURBS_COLOR: FUNCTION_TYPE( 
			None, ctypes.POINTER(simple.GLfloat)
		),
		simple.GLU_NURBS_COLOR_DATA: FUNCTION_TYPE( 
			None, ctypes.POINTER(simple.GLfloat), ctypes.POINTER(simple.GLvoid) 
		),
		simple.GLU_NURBS_TEXTURE_COORD: FUNCTION_TYPE( 
			None, ctypes.POINTER(simple.GLfloat)
		),
		simple.GLU_NURBS_TEXTURE_COORD_DATA: FUNCTION_TYPE( 
			None, ctypes.POINTER(simple.GLfloat), ctypes.POINTER(simple.GLvoid) 
		),
		simple.GLU_NURBS_END:FUNCTION_TYPE( 
			None
		),
		simple.GLU_NURBS_END_DATA: FUNCTION_TYPE( 
			None, ctypes.POINTER(simple.GLvoid) 
		),
		simple.GLU_NURBS_ERROR:FUNCTION_TYPE( 
			None, simple.GLenum, 
		),
	}
	WRAPPER_METHODS = {
		simple.GLU_NURBS_BEGIN: None,
		simple.GLU_NURBS_BEGIN_DATA: '_justOOR',
		simple.GLU_NURBS_VERTEX: '_vec3',
		simple.GLU_NURBS_VERTEX_DATA: '_vec3',
		simple.GLU_NURBS_NORMAL: '_vec3',
		simple.GLU_NURBS_NORMAL_DATA: '_vec3',
		simple.GLU_NURBS_COLOR: '_vec4',
		simple.GLU_NURBS_COLOR_DATA: '_vec4',
		simple.GLU_NURBS_TEXTURE_COORD: '_tex',
		simple.GLU_NURBS_TEXTURE_COORD_DATA: '_tex',
		simple.GLU_NURBS_END: None,
		simple.GLU_NURBS_END_DATA: '_justOOR',
		simple.GLU_NURBS_ERROR: None,
	}
	def _justOOR( self, function ):
		"""Just do OOR on the last argument..."""
		def getOOR( *args ):
			args = args[:-1] + (self.originalObject(args[-1]),)
			return function( *args )
		return getOOR
	def _vec3( self, function, size=3 ):
		"""Convert first arg to size-element array, do OOR on arg2 if present"""
		def vec( *args ):
			vec = self.ptrAsArray(args[0],size,arrays.GLfloatArray)
			if len(args) > 1:
				oor = self.originalObject(args[1])
				return function( vec, oor )
			else:
				return function( vec )
		return vec
	def _vec4( self, function ):
		"""Size-4 vector version..."""
		return self._vec3( function, 4 )
	def _tex( self, function ):
		"""Texture coordinate callback 
		
		NOTE: there is no way for *us* to tell what size the array is, you will 
		get back a raw data-point, not an array, as you do for all other callback 
		types!!!
		"""
		def oor( *args ):
			if len(args) > 1:
				oor = self.originalObject(args[1])
				return function( args[0], oor )
			else:
				return function( args[0] )
		return oor

# XXX yes, this is a side-effect...
simple.gluNewNurbsRenderer.restype = ctypes.POINTER( GLUnurbs )

def _callbackWithType( funcType ):
	"""Get gluNurbsCallback function with set last arg-type"""
	result =  platform.copyBaseFunction(
		simple.gluNurbsCallback
	)
	result.argtypes = [ctypes.POINTER(GLUnurbs), simple.GLenum, funcType]
	assert result.argtypes[-1] == funcType
	return result

for (c,funcType) in GLUnurbs.CALLBACK_TYPES.items():
	cb = _callbackWithType( funcType )
	GLUnurbs.CALLBACK_FUNCTION_REGISTRARS[ c ] = cb
	assert funcType == GLUnurbs.CALLBACK_TYPES[c]
	assert cb.argtypes[-1] == funcType
del c,cb, funcType

def gluNurbsCallback( nurb, which, CallBackFunc ):
	"""Dispatch to the nurb's addCallback operation"""
	return nurb.addCallback( which, CallBackFunc )

@lazy( simple.gluNewNurbsRenderer )
def gluNewNurbsRenderer( baseFunction ):
	"""Return a new nurbs renderer for the system (dereferences pointer)"""
	newSet = baseFunction()
	new = newSet[0]
	#new.__class__ = GLUnurbs # yes, I know, ick
	return new

@lazy( simple.gluNurbsCallbackData )
def gluNurbsCallbackData( baseFunction, nurb, userData ):
	"""Note the Python object for use as userData by the nurb"""
	return baseFunction( 
		nurb, nurb.noteObject( userData ) 
	)

@lazy( simple.gluNurbsCallbackDataEXT )
def gluNurbsCallbackDataEXT( baseFunction,nurb, userData ):
	"""Note the Python object for use as userData by the nurb"""
	return baseFunction( 
		nurb, nurb.noteObject( userData ) 
	)

@lazy( simple.gluNurbsCurve )
def gluNurbsCurve( baseFunction, nurb, knots, control, type ):
	"""Pythonic version of gluNurbsCurve
	
	Calculates knotCount, stride, and order automatically
	"""
	knots = arrays.GLfloatArray.asArray( knots )
	knotCount = arrays.GLfloatArray.arraySize( knots )
	control = arrays.GLfloatArray.asArray( control )
	length,step = arrays.GLfloatArray.dimensions( control )
	order = knotCount - length
	return baseFunction(
		nurb, knotCount, knots, step, control, order, type,
	)

@lazy( simple.gluNurbsSurface )
def gluNurbsSurface( baseFunction, nurb, sKnots, tKnots, control, type ):
	"""Pythonic version of gluNurbsSurface
	
	Calculates knotCount, stride, and order automatically
	"""
	sKnots = arrays.GLfloatArray.asArray( sKnots )
	sKnotCount = arrays.GLfloatArray.arraySize( sKnots )
	tKnots = arrays.GLfloatArray.asArray( tKnots )
	tKnotCount = arrays.GLfloatArray.arraySize( tKnots )
	control = arrays.GLfloatArray.asArray( control )

	length,width,step = arrays.GLfloatArray.dimensions( control )
	sOrder = sKnotCount - length 
	tOrder = tKnotCount - width 
	sStride = width*step
	tStride = step
	
	assert (sKnotCount-sOrder)*(tKnotCount-tOrder) == length*width, (
		nurb, sKnotCount, sKnots, tKnotCount, tKnots,
		sStride, tStride, control,
		sOrder,tOrder,
		type
	)

	result = baseFunction(
		nurb, sKnotCount, sKnots, tKnotCount, tKnots,
		sStride, tStride, control,
		sOrder,tOrder,
		type
	)
	return result

@lazy( simple.gluPwlCurve )
def gluPwlCurve( baseFunction, nurb, data, type ):
	"""gluPwlCurve -- piece-wise linear curve within GLU context
	
	data -- the data-array 
	type -- determines number of elements/data-point
	"""
	data = arrays.GLfloatArray.asArray( data )
	if type == simple.GLU_MAP1_TRIM_2:
		divisor = 2
	elif type == simple.GLU_MAP_TRIM_3:
		divisor = 3
	else:
		raise ValueError( """Unrecognised type constant: %s"""%(type))
	size = arrays.GLfloatArray.arraySize( data )
	size = int(size/divisor)
	return baseFunction( nurb, size, data, divisor, type )



'''OpenGL extension SGIX.polynomial_ffd

The official definition of this extension is available here:
	http://oss.sgi.com/projects/ogl-sample/registry/SGIX/polynomial_ffd.txt

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_SGIX_polynomial_ffd'
GL_GEOMETRY_DEFORMATION_SGIX = constant.Constant( 'GL_GEOMETRY_DEFORMATION_SGIX', 0x8194 )
GL_TEXTURE_DEFORMATION_SGIX = constant.Constant( 'GL_TEXTURE_DEFORMATION_SGIX', 0x8195 )
GL_DEFORMATIONS_MASK_SGIX = constant.Constant( 'GL_DEFORMATIONS_MASK_SGIX', 0x8196 )
GL_MAX_DEFORMATION_ORDER_SGIX = constant.Constant( 'GL_MAX_DEFORMATION_ORDER_SGIX', 0x8197 )
glDeformationMap3dSGIX = platform.createExtensionFunction( 
	'glDeformationMap3dSGIX', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, constants.GLdouble, constants.GLdouble, constants.GLint, constants.GLint, constants.GLdouble, constants.GLdouble, constants.GLint, constants.GLint, constants.GLdouble, constants.GLdouble, constants.GLint, constants.GLint, arrays.GLdoubleArray,),
	doc = 'glDeformationMap3dSGIX( GLenum(target), GLdouble(u1), GLdouble(u2), GLint(ustride), GLint(uorder), GLdouble(v1), GLdouble(v2), GLint(vstride), GLint(vorder), GLdouble(w1), GLdouble(w2), GLint(wstride), GLint(worder), GLdoubleArray(points) ) -> None',
	argNames = ('target', 'u1', 'u2', 'ustride', 'uorder', 'v1', 'v2', 'vstride', 'vorder', 'w1', 'w2', 'wstride', 'worder', 'points',),
)

glDeformationMap3fSGIX = platform.createExtensionFunction( 
	'glDeformationMap3fSGIX', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, constants.GLfloat, constants.GLfloat, constants.GLint, constants.GLint, constants.GLfloat, constants.GLfloat, constants.GLint, constants.GLint, constants.GLfloat, constants.GLfloat, constants.GLint, constants.GLint, arrays.GLfloatArray,),
	doc = 'glDeformationMap3fSGIX( GLenum(target), GLfloat(u1), GLfloat(u2), GLint(ustride), GLint(uorder), GLfloat(v1), GLfloat(v2), GLint(vstride), GLint(vorder), GLfloat(w1), GLfloat(w2), GLint(wstride), GLint(worder), GLfloatArray(points) ) -> None',
	argNames = ('target', 'u1', 'u2', 'ustride', 'uorder', 'v1', 'v2', 'vstride', 'vorder', 'w1', 'w2', 'wstride', 'worder', 'points',),
)

glDeformSGIX = platform.createExtensionFunction( 
	'glDeformSGIX', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLbitfield,),
	doc = 'glDeformSGIX( GLbitfield(mask) ) -> None',
	argNames = ('mask',),
)

glLoadIdentityDeformationMapSGIX = platform.createExtensionFunction( 
	'glLoadIdentityDeformationMapSGIX', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLbitfield,),
	doc = 'glLoadIdentityDeformationMapSGIX( GLbitfield(mask) ) -> None',
	argNames = ('mask',),
)


def glInitPolynomialFfdSGIX():
	'''Return boolean indicating whether this extension is available'''
	return extensions.hasGLExtension( EXTENSION_NAME )



'''OpenGL extension EXT.abgr

Overview (from the spec)
	
	EXT_abgr extends the list of host-memory color formats.  Specifically,
	it provides a reverse-order alternative to image format RGBA.  The ABGR
	component order matches the cpack Iris GL format on big-endian machines.

The official definition of this extension is available here:
	http://oss.sgi.com/projects/ogl-sample/registry/EXT/abgr.txt

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_EXT_abgr'
GL_ABGR_EXT = constant.Constant( 'GL_ABGR_EXT', 0x8000 )


def glInitAbgrEXT():
	'''Return boolean indicating whether this extension is available'''
	return extensions.hasGLExtension( EXTENSION_NAME )



import sys, time, math
import pygame

from OpenGL.GL import *
from OpenGL.GLU import *

FRAME_TIME = 0.02
colors = []
vertexes = []

def setup_scene(screen):
	glViewport(0, 0, screen.get_width(), screen.get_height())
	glMatrixMode(GL_PROJECTION)
	glLoadIdentity()
	gluOrtho2D(-4, 4, -3, 3)
	
	glEnableClientState(GL_COLOR_ARRAY)
	glEnableClientState(GL_VERTEX_ARRAY)
	
	radius = 2.0
	
	for i in range(10):
		angle1 = math.pi* i/ 5.0 
		angle2 = math.pi* (i+ 1)/ 5.0
		
		vertexes.append([radius* math.cos(angle1), radius* math.sin(angle1), 0.0])
		vertexes.append([radius* math.cos(angle2), radius* math.sin(angle2), 0.0])
		vertexes.append([0.0, 0.0, 0.0])
		
		colors.append([1.0, 0.0, 0.0, 1.0])
		colors.append([1.0, 0.0, 0.0, 1.0])
		colors.append([0.5, 0.0, 0.0, 1.0])
		
	print len(vertexes), vertexes
	print len(colors), colors
		
	glVertexPointerf(vertexes)
	glColorPointerf(colors)
	
def render_scene():
	glClear(GL_COLOR_BUFFER_BIT)
	glMatrixMode(GL_MODELVIEW)
	glLoadIdentity()
	
	t = time.time()/ 4
	subtime = t- math.floor(t) 
	glRotatef(subtime* 360, 0, 0, 1)
	
	glDrawArrays(GL_TRIANGLES, 0, 30)

if __name__ == "__main__":
	pygame.init()

	screen = pygame.display.set_mode([800, 600], pygame.OPENGL | pygame.DOUBLEBUF)
	pygame.display.set_caption('Python OpenGL test', 'pygame')

	setup_scene(screen)

	while True:
		starttime = time.time()
		
		for event in pygame.event.get():
			if event.type == pygame.QUIT:
				sys.exit()
			if event.type == pygame.KEYDOWN:
				if event.key == pygame.K_ESCAPE:
					sys.exit()
		
		render_scene()
		pygame.display.flip()
		
		timeleft = FRAME_TIME- (time.time()- starttime)
		if (timeleft > 0):
			time.sleep(timeleft)



'''OpenGL extension SGIS.sharpen_texture

Overview (from the spec)
	
	This extension introduces texture magnification filters that sharpen
	the resulting image by extrapolating from the level 1 image to the
	level 0 image.  Sharpening can be enabled for all color channels, for
	the alpha channel only, or for the red, green, and blue channels only.

The official definition of this extension is available here:
	http://oss.sgi.com/projects/ogl-sample/registry/SGIS/sharpen_texture.txt

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_SGIS_sharpen_texture'
GL_LINEAR_SHARPEN_SGIS = constant.Constant( 'GL_LINEAR_SHARPEN_SGIS', 0x80AD )
GL_LINEAR_SHARPEN_ALPHA_SGIS = constant.Constant( 'GL_LINEAR_SHARPEN_ALPHA_SGIS', 0x80AE )
GL_LINEAR_SHARPEN_COLOR_SGIS = constant.Constant( 'GL_LINEAR_SHARPEN_COLOR_SGIS', 0x80AF )
GL_SHARPEN_TEXTURE_FUNC_POINTS_SGIS = constant.Constant( 'GL_SHARPEN_TEXTURE_FUNC_POINTS_SGIS', 0x80B0 )
glSharpenTexFuncSGIS = platform.createExtensionFunction( 
	'glSharpenTexFuncSGIS', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, constants.GLsizei, arrays.GLfloatArray,),
	doc = 'glSharpenTexFuncSGIS( GLenum(target), GLsizei(n), GLfloatArray(points) ) -> None',
	argNames = ('target', 'n', 'points',),
)

glGetSharpenTexFuncSGIS = platform.createExtensionFunction( 
	'glGetSharpenTexFuncSGIS', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, arrays.GLfloatArray,),
	doc = 'glGetSharpenTexFuncSGIS( GLenum(target), GLfloatArray(points) ) -> None',
	argNames = ('target', 'points',),
)


def glInitSharpenTextureSGIS():
	'''Return boolean indicating whether this extension is available'''
	return extensions.hasGLExtension( EXTENSION_NAME )



'''OpenGL extension ARB.shader_objects

This module customises the behaviour of the 
OpenGL.raw.GL.ARB.shader_objects to provide a more 
Python-friendly API
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions, wrapper
from OpenGL.GL import glget
import ctypes
from OpenGL.raw.GL.ARB.shader_objects import *
### END AUTOGENERATED SECTION
EXTENSION_NAME = 'GL_ARB_shader_objects'
import OpenGL
from OpenGL import converters, error
GL_INFO_LOG_LENGTH_ARB = constant.Constant( 'GL_INFO_LOG_LENGTH_ARB', 0x8B84 )

glShaderSourceARB = platform.createExtensionFunction( 
	'glShaderSourceARB', dll=platform.GL,
	resultType=None, 
	argTypes=(constants.GLhandleARB, constants.GLsizei, ctypes.POINTER(ctypes.c_char_p), arrays.GLintArray,),
	doc = 'glShaderSourceARB( GLhandleARB(shaderObj), str( string) ) -> None',
	argNames = ('shaderObj', 'count', 'string', 'length',),
	extension = EXTENSION_NAME,
)
conv = converters.StringLengths( name='string' )
glShaderSourceARB = wrapper.wrapper(
	glShaderSourceARB
).setPyConverter(
	'count' # number of strings
).setPyConverter( 
	'length' # lengths of strings
).setPyConverter(
	'string', conv.stringArray
).setCResolver(
	'string', conv.stringArrayForC,
).setCConverter(
	'length', conv,
).setCConverter(
	'count', conv.totalCount,
)
del conv

for size in (1,2,3,4):
	for format,arrayType in (
		('f',arrays.GLfloatArray),
		('i',arrays.GLintArray),
	):
		name = 'glUniform%(size)s%(format)svARB'%globals()
		globals()[name] = arrays.setInputArraySizeType(
			globals()[name],
			size,
			arrayType, 
			'value',
		)
		del format, arrayType
	del size

base_glGetObjectParameterivARB = glGetObjectParameterivARB
def glGetObjectParameterivARB( shader, pname ):
	"""Retrieve the integer parameter for the given shader"""
	status = arrays.GLintArray.zeros( (1,))
	status[0] = 1 
	base_glGetObjectParameterivARB(
		shader, pname, status
	)
	return status[0]
glGetObjectParameterivARB.wrappedOperation = base_glGetObjectParameterivARB
base_glGetObjectParameterfvARB = glGetObjectParameterfvARB
def glGetObjectParameterfvARB( shader, pname ):
	"""Retrieve the float parameter for the given shader"""
	status = arrays.GLfloatArray.zeros( (1,))
	status[0] = 1.0
	base_glGetObjectParameterfvARB(
		shader, pname,status
	)
	return status[0]
glGetObjectParameterfvARB.wrappedOperation = base_glGetObjectParameterfvARB
def _afterCheck( key ):
	"""Generate an error-checking function for compilation operations"""
	def GLSLCheckError( 
		result,
		baseOperation=None,
		cArguments=None,
		*args
	):
		result = error.glCheckError( result, baseOperation, cArguments, *args )
		status = glGetObjectParameterivARB(
			cArguments[0], key
		)
		if not status:
			raise error.GLError( 
				result = result,
				baseOperation = baseOperation,
				cArguments = cArguments,
				description= glGetInfoLogARB( cArguments[0] )
			)
		return result
	return GLSLCheckError

if OpenGL.ERROR_CHECKING:
	glCompileShaderARB.errcheck = _afterCheck( GL_OBJECT_COMPILE_STATUS_ARB )
if OpenGL.ERROR_CHECKING:
	glLinkProgramARB.errcheck = _afterCheck( GL_OBJECT_LINK_STATUS_ARB )
## Not sure why, but these give invalid operation :(
##if glValidateProgramARB and OpenGL.ERROR_CHECKING:
##	glValidateProgramARB.errcheck = _afterCheck( GL_OBJECT_VALIDATE_STATUS_ARB )

base_glGetInfoLogARB = glGetInfoLogARB
def glGetInfoLogARB( obj ):
	"""Retrieve the program/shader's error messages as a Python string
	
	returns string which is '' if no message
	"""
	length = int(glGetObjectParameterivARB(obj, GL_INFO_LOG_LENGTH_ARB))
	if length > 0:
		log = ctypes.create_string_buffer(length)
		base_glGetInfoLogARB(obj, length, None, log)
		return log.value.strip('\000') # null-termination
	return ''
glGetInfoLogARB.wrappedOperation = base_glGetInfoLogARB

base_glGetAttachedObjectsARB = glGetAttachedObjectsARB
def glGetAttachedObjectsARB( obj ):
	"""Retrieve the attached objects as an array of GLhandleARB instances"""
	length= glGetObjectParameterivARB( obj, GL_OBJECT_ATTACHED_OBJECTS_ARB )
	if length > 0:
		storage = arrays.GLuintArray.zeros( (length,))
		base_glGetAttachedObjectsARB( obj, length, None, storage )
		return storage
	return arrays.GLuintArray.zeros( (0,))
glGetAttachedObjectsARB.wrappedOperation = base_glGetAttachedObjectsARB

base_glGetShaderSourceARB = glGetShaderSourceARB
def glGetShaderSourceARB( obj ):
	"""Retrieve the program/shader's source code as a Python string
	
	returns string which is '' if no source code
	"""
	length = int(glGetObjectParameterivARB(obj, GL_OBJECT_SHADER_SOURCE_LENGTH_ARB))
	if length > 0:
		source = ctypes.create_string_buffer(length)
		base_glGetShaderSourceARB(obj, length, None, source)
		return source.value.strip('\000') # null-termination
	return ''
glGetShaderSourceARB.wrappedOperation = base_glGetShaderSourceARB

base_glGetActiveUniformARB = glGetActiveUniformARB
def glGetActiveUniformARB(program, index):
	"""Retrieve the name, size and type of the uniform of the index in the program"""
	max_index = int(glGetObjectParameterivARB( program, GL_OBJECT_ACTIVE_UNIFORMS_ARB ))
	length = int(glGetObjectParameterivARB( program, GL_OBJECT_ACTIVE_UNIFORM_MAX_LENGTH_ARB))
	if index < max_index and index >= 0 and length > 0:
		name = ctypes.create_string_buffer(length)
		size = arrays.GLintArray.zeros( (1,))
		gl_type = arrays.GLuintArray.zeros( (1,))
		base_glGetActiveUniformARB(program, index, length, None, size, gl_type, name)
		return name.value, size[0], gl_type[0]
	raise IndexError, 'Index %s out of range 0 to %i' % (index, max_index - 1, )
glGetActiveUniformARB.wrappedOperation = base_glGetActiveUniformARB



# -*- coding: iso-8859-1 -*-
"""Raw (C-style) API for OpenGL.GLU

Automatically generated by the generateraw script, do not edit!
"""
from OpenGL.raw.GLU.constants import *

from ctypes import *
from OpenGL import platform, arrays
from OpenGL.constant import Constant
from OpenGL import constants as GLconstants
GLvoid = GLconstants.GLvoid

FUNCTION_TYPE = platform.PLATFORM.functionTypeFor( platform.GLU )
from OpenGL.constants import GLint
from OpenGL.constants import GLenum
from OpenGL.constants import GLsizei
from OpenGL.constants import GLboolean
from OpenGL.constants import GLubyte
from OpenGL.constants import GLdouble
from OpenGL.constants import GLfloat


class GLUnurbs(Structure):
    pass
GLUnurbs._fields_ = [
    # /usr/include/GL/glu.h 257
]
GLUnurbsObj = GLUnurbs
class GLUquadric(Structure):
    pass
GLUquadric._fields_ = [
    # /usr/include/GL/glu.h 258
]
GLUquadricObj = GLUquadric
class GLUtesselator(Structure):
    pass
GLUtesselator._fields_ = [
    # /usr/include/GL/glu.h 259
]
GLUtesselatorObj = GLUtesselator
GLUtriangulatorObj = GLUtesselator


# /usr/include/GL/glu.h 276
gluBeginCurve = platform.createBaseFunction( 
	'gluBeginCurve', dll=platform.GLU, resultType=None, 
	argTypes=[POINTER(GLUnurbs)],
	doc='gluBeginCurve( POINTER(GLUnurbs)(nurb) ) -> None', 
	argNames=('nurb',),
)


# /usr/include/GL/glu.h 277
gluBeginPolygon = platform.createBaseFunction( 
	'gluBeginPolygon', dll=platform.GLU, resultType=None, 
	argTypes=[POINTER(GLUtesselator)],
	doc='gluBeginPolygon( POINTER(GLUtesselator)(tess) ) -> None', 
	argNames=('tess',),
)


# /usr/include/GL/glu.h 278
gluBeginSurface = platform.createBaseFunction( 
	'gluBeginSurface', dll=platform.GLU, resultType=None, 
	argTypes=[POINTER(GLUnurbs)],
	doc='gluBeginSurface( POINTER(GLUnurbs)(nurb) ) -> None', 
	argNames=('nurb',),
)


# /usr/include/GL/glu.h 279
gluBeginTrim = platform.createBaseFunction( 
	'gluBeginTrim', dll=platform.GLU, resultType=None, 
	argTypes=[POINTER(GLUnurbs)],
	doc='gluBeginTrim( POINTER(GLUnurbs)(nurb) ) -> None', 
	argNames=('nurb',),
)


# /usr/include/GL/glu.h 280
gluBuild1DMipmapLevels = platform.createBaseFunction( 
	'gluBuild1DMipmapLevels', dll=platform.GLU, resultType=GLint, 
	argTypes=[GLenum,GLint,GLsizei,GLenum,GLenum,GLint,GLint,GLint,c_void_p],
	doc='gluBuild1DMipmapLevels( GLenum(target), GLint(internalFormat), GLsizei(width), GLenum(format), GLenum(type), GLint(level), GLint(base), GLint(max), c_void_p(data) ) -> GLint', 
	argNames=('target', 'internalFormat', 'width', 'format', 'type', 'level', 'base', 'max', 'data'),
)


# /usr/include/GL/glu.h 281
gluBuild1DMipmaps = platform.createBaseFunction( 
	'gluBuild1DMipmaps', dll=platform.GLU, resultType=GLint, 
	argTypes=[GLenum,GLint,GLsizei,GLenum,GLenum,c_void_p],
	doc='gluBuild1DMipmaps( GLenum(target), GLint(internalFormat), GLsizei(width), GLenum(format), GLenum(type), c_void_p(data) ) -> GLint', 
	argNames=('target', 'internalFormat', 'width', 'format', 'type', 'data'),
)


# /usr/include/GL/glu.h 282
gluBuild2DMipmapLevels = platform.createBaseFunction( 
	'gluBuild2DMipmapLevels', dll=platform.GLU, resultType=GLint, 
	argTypes=[GLenum,GLint,GLsizei,GLsizei,GLenum,GLenum,GLint,GLint,GLint,c_void_p],
	doc='gluBuild2DMipmapLevels( GLenum(target), GLint(internalFormat), GLsizei(width), GLsizei(height), GLenum(format), GLenum(type), GLint(level), GLint(base), GLint(max), c_void_p(data) ) -> GLint', 
	argNames=('target', 'internalFormat', 'width', 'height', 'format', 'type', 'level', 'base', 'max', 'data'),
)


# /usr/include/GL/glu.h 283
gluBuild2DMipmaps = platform.createBaseFunction( 
	'gluBuild2DMipmaps', dll=platform.GLU, resultType=GLint, 
	argTypes=[GLenum,GLint,GLsizei,GLsizei,GLenum,GLenum,c_void_p],
	doc='gluBuild2DMipmaps( GLenum(target), GLint(internalFormat), GLsizei(width), GLsizei(height), GLenum(format), GLenum(type), c_void_p(data) ) -> GLint', 
	argNames=('target', 'internalFormat', 'width', 'height', 'format', 'type', 'data'),
)


# /usr/include/GL/glu.h 284
gluBuild3DMipmapLevels = platform.createBaseFunction( 
	'gluBuild3DMipmapLevels', dll=platform.GLU, resultType=GLint, 
	argTypes=[GLenum,GLint,GLsizei,GLsizei,GLsizei,GLenum,GLenum,GLint,GLint,GLint,c_void_p],
	doc='gluBuild3DMipmapLevels( GLenum(target), GLint(internalFormat), GLsizei(width), GLsizei(height), GLsizei(depth), GLenum(format), GLenum(type), GLint(level), GLint(base), GLint(max), c_void_p(data) ) -> GLint', 
	argNames=('target', 'internalFormat', 'width', 'height', 'depth', 'format', 'type', 'level', 'base', 'max', 'data'),
)


# /usr/include/GL/glu.h 285
gluBuild3DMipmaps = platform.createBaseFunction( 
	'gluBuild3DMipmaps', dll=platform.GLU, resultType=GLint, 
	argTypes=[GLenum,GLint,GLsizei,GLsizei,GLsizei,GLenum,GLenum,c_void_p],
	doc='gluBuild3DMipmaps( GLenum(target), GLint(internalFormat), GLsizei(width), GLsizei(height), GLsizei(depth), GLenum(format), GLenum(type), c_void_p(data) ) -> GLint', 
	argNames=('target', 'internalFormat', 'width', 'height', 'depth', 'format', 'type', 'data'),
)


# /usr/include/GL/glu.h 286
gluCheckExtension = platform.createBaseFunction( 
	'gluCheckExtension', dll=platform.GLU, resultType=GLboolean, 
	argTypes=[arrays.GLubyteArray,arrays.GLubyteArray],
	doc='gluCheckExtension( arrays.GLubyteArray(extName), arrays.GLubyteArray(extString) ) -> GLboolean', 
	argNames=('extName', 'extString'),
)


# /usr/include/GL/glu.h 287
gluCylinder = platform.createBaseFunction( 
	'gluCylinder', dll=platform.GLU, resultType=None, 
	argTypes=[POINTER(GLUquadric),GLdouble,GLdouble,GLdouble,GLint,GLint],
	doc='gluCylinder( POINTER(GLUquadric)(quad), GLdouble(base), GLdouble(top), GLdouble(height), GLint(slices), GLint(stacks) ) -> None', 
	argNames=('quad', 'base', 'top', 'height', 'slices', 'stacks'),
)


# /usr/include/GL/glu.h 288
gluDeleteNurbsRenderer = platform.createBaseFunction( 
	'gluDeleteNurbsRenderer', dll=platform.GLU, resultType=None, 
	argTypes=[POINTER(GLUnurbs)],
	doc='gluDeleteNurbsRenderer( POINTER(GLUnurbs)(nurb) ) -> None', 
	argNames=('nurb',),
)


# /usr/include/GL/glu.h 289
gluDeleteQuadric = platform.createBaseFunction( 
	'gluDeleteQuadric', dll=platform.GLU, resultType=None, 
	argTypes=[POINTER(GLUquadric)],
	doc='gluDeleteQuadric( POINTER(GLUquadric)(quad) ) -> None', 
	argNames=('quad',),
)


# /usr/include/GL/glu.h 290
gluDeleteTess = platform.createBaseFunction( 
	'gluDeleteTess', dll=platform.GLU, resultType=None, 
	argTypes=[POINTER(GLUtesselator)],
	doc='gluDeleteTess( POINTER(GLUtesselator)(tess) ) -> None', 
	argNames=('tess',),
)


# /usr/include/GL/glu.h 291
gluDisk = platform.createBaseFunction( 
	'gluDisk', dll=platform.GLU, resultType=None, 
	argTypes=[POINTER(GLUquadric),GLdouble,GLdouble,GLint,GLint],
	doc='gluDisk( POINTER(GLUquadric)(quad), GLdouble(inner), GLdouble(outer), GLint(slices), GLint(loops) ) -> None', 
	argNames=('quad', 'inner', 'outer', 'slices', 'loops'),
)


# /usr/include/GL/glu.h 292
gluEndCurve = platform.createBaseFunction( 
	'gluEndCurve', dll=platform.GLU, resultType=None, 
	argTypes=[POINTER(GLUnurbs)],
	doc='gluEndCurve( POINTER(GLUnurbs)(nurb) ) -> None', 
	argNames=('nurb',),
)


# /usr/include/GL/glu.h 293
gluEndPolygon = platform.createBaseFunction( 
	'gluEndPolygon', dll=platform.GLU, resultType=None, 
	argTypes=[POINTER(GLUtesselator)],
	doc='gluEndPolygon( POINTER(GLUtesselator)(tess) ) -> None', 
	argNames=('tess',),
)


# /usr/include/GL/glu.h 294
gluEndSurface = platform.createBaseFunction( 
	'gluEndSurface', dll=platform.GLU, resultType=None, 
	argTypes=[POINTER(GLUnurbs)],
	doc='gluEndSurface( POINTER(GLUnurbs)(nurb) ) -> None', 
	argNames=('nurb',),
)


# /usr/include/GL/glu.h 295
gluEndTrim = platform.createBaseFunction( 
	'gluEndTrim', dll=platform.GLU, resultType=None, 
	argTypes=[POINTER(GLUnurbs)],
	doc='gluEndTrim( POINTER(GLUnurbs)(nurb) ) -> None', 
	argNames=('nurb',),
)


# /usr/include/GL/glu.h 296
gluErrorString = platform.createBaseFunction( 
	'gluErrorString', dll=platform.GLU, resultType=POINTER(GLubyte), 
	argTypes=[GLenum],
	doc='gluErrorString( GLenum(error) ) -> POINTER(GLubyte)', 
	argNames=('error',),
)


# /usr/include/GL/glu.h 297
gluGetNurbsProperty = platform.createBaseFunction( 
	'gluGetNurbsProperty', dll=platform.GLU, resultType=None, 
	argTypes=[POINTER(GLUnurbs),GLenum,arrays.GLfloatArray],
	doc='gluGetNurbsProperty( POINTER(GLUnurbs)(nurb), GLenum(property), arrays.GLfloatArray(data) ) -> None', 
	argNames=('nurb', 'property', 'data'),
)


# /usr/include/GL/glu.h 298
gluGetString = platform.createBaseFunction( 
	'gluGetString', dll=platform.GLU, resultType=POINTER(GLubyte), 
	argTypes=[GLenum],
	doc='gluGetString( GLenum(name) ) -> POINTER(GLubyte)', 
	argNames=('name',),
)


# /usr/include/GL/glu.h 299
gluGetTessProperty = platform.createBaseFunction( 
	'gluGetTessProperty', dll=platform.GLU, resultType=None, 
	argTypes=[POINTER(GLUtesselator),GLenum,arrays.GLdoubleArray],
	doc='gluGetTessProperty( POINTER(GLUtesselator)(tess), GLenum(which), arrays.GLdoubleArray(data) ) -> None', 
	argNames=('tess', 'which', 'data'),
)


# /usr/include/GL/glu.h 300
gluLoadSamplingMatrices = platform.createBaseFunction( 
	'gluLoadSamplingMatrices', dll=platform.GLU, resultType=None, 
	argTypes=[POINTER(GLUnurbs),arrays.GLfloatArray,arrays.GLfloatArray,arrays.GLintArray],
	doc='gluLoadSamplingMatrices( POINTER(GLUnurbs)(nurb), arrays.GLfloatArray(model), arrays.GLfloatArray(perspective), arrays.GLintArray(view) ) -> None', 
	argNames=('nurb', 'model', 'perspective', 'view'),
)


# /usr/include/GL/glu.h 301
gluLookAt = platform.createBaseFunction( 
	'gluLookAt', dll=platform.GLU, resultType=None, 
	argTypes=[GLdouble,GLdouble,GLdouble,GLdouble,GLdouble,GLdouble,GLdouble,GLdouble,GLdouble],
	doc='gluLookAt( GLdouble(eyeX), GLdouble(eyeY), GLdouble(eyeZ), GLdouble(centerX), GLdouble(centerY), GLdouble(centerZ), GLdouble(upX), GLdouble(upY), GLdouble(upZ) ) -> None', 
	argNames=('eyeX', 'eyeY', 'eyeZ', 'centerX', 'centerY', 'centerZ', 'upX', 'upY', 'upZ'),
)


# /usr/include/GL/glu.h 302
gluNewNurbsRenderer = platform.createBaseFunction( 
	'gluNewNurbsRenderer', dll=platform.GLU, resultType=POINTER(GLUnurbs), 
	argTypes=[],
	doc='gluNewNurbsRenderer(  ) -> POINTER(GLUnurbs)', 
	argNames=(),
)


# /usr/include/GL/glu.h 303
gluNewQuadric = platform.createBaseFunction( 
	'gluNewQuadric', dll=platform.GLU, resultType=POINTER(GLUquadric), 
	argTypes=[],
	doc='gluNewQuadric(  ) -> POINTER(GLUquadric)', 
	argNames=(),
)


# /usr/include/GL/glu.h 304
gluNewTess = platform.createBaseFunction( 
	'gluNewTess', dll=platform.GLU, resultType=POINTER(GLUtesselator), 
	argTypes=[],
	doc='gluNewTess(  ) -> POINTER(GLUtesselator)', 
	argNames=(),
)


# /usr/include/GL/glu.h 305
gluNextContour = platform.createBaseFunction( 
	'gluNextContour', dll=platform.GLU, resultType=None, 
	argTypes=[POINTER(GLUtesselator),GLenum],
	doc='gluNextContour( POINTER(GLUtesselator)(tess), GLenum(type) ) -> None', 
	argNames=('tess', 'type'),
)


_GLUfuncptr = FUNCTION_TYPE(None)
# /usr/include/GL/glu.h 306
gluNurbsCallback = platform.createBaseFunction( 
	'gluNurbsCallback', dll=platform.GLU, resultType=None, 
	argTypes=[POINTER(GLUnurbs),GLenum,_GLUfuncptr],
	doc='gluNurbsCallback( POINTER(GLUnurbs)(nurb), GLenum(which), _GLUfuncptr(CallBackFunc) ) -> None', 
	argNames=('nurb', 'which', 'CallBackFunc'),
)

GLvoid = None
# /usr/include/GL/glu.h 307
gluNurbsCallbackData = platform.createBaseFunction( 
	'gluNurbsCallbackData', dll=platform.GLU, resultType=None, 
	argTypes=[POINTER(GLUnurbs),POINTER(GLvoid)],
	doc='gluNurbsCallbackData( POINTER(GLUnurbs)(nurb), POINTER(GLvoid)(userData) ) -> None', 
	argNames=('nurb', 'userData'),
)


# /usr/include/GL/glu.h 308
gluNurbsCallbackDataEXT = platform.createBaseFunction( 
	'gluNurbsCallbackDataEXT', dll=platform.GLU, resultType=None, 
	argTypes=[POINTER(GLUnurbs),POINTER(GLvoid)],
	doc='gluNurbsCallbackDataEXT( POINTER(GLUnurbs)(nurb), POINTER(GLvoid)(userData) ) -> None', 
	argNames=('nurb', 'userData'),
)


# /usr/include/GL/glu.h 309
gluNurbsCurve = platform.createBaseFunction( 
	'gluNurbsCurve', dll=platform.GLU, resultType=None, 
	argTypes=[POINTER(GLUnurbs),GLint,arrays.GLfloatArray,GLint,arrays.GLfloatArray,GLint,GLenum],
	doc='gluNurbsCurve( POINTER(GLUnurbs)(nurb), GLint(knotCount), arrays.GLfloatArray(knots), GLint(stride), arrays.GLfloatArray(control), GLint(order), GLenum(type) ) -> None', 
	argNames=('nurb', 'knotCount', 'knots', 'stride', 'control', 'order', 'type'),
)


# /usr/include/GL/glu.h 310
gluNurbsProperty = platform.createBaseFunction( 
	'gluNurbsProperty', dll=platform.GLU, resultType=None, 
	argTypes=[POINTER(GLUnurbs),GLenum,GLfloat],
	doc='gluNurbsProperty( POINTER(GLUnurbs)(nurb), GLenum(property), GLfloat(value) ) -> None', 
	argNames=('nurb', 'property', 'value'),
)


# /usr/include/GL/glu.h 311
gluNurbsSurface = platform.createBaseFunction( 
	'gluNurbsSurface', dll=platform.GLU, resultType=None, 
	argTypes=[POINTER(GLUnurbs),GLint,arrays.GLfloatArray,GLint,arrays.GLfloatArray,GLint,GLint,arrays.GLfloatArray,GLint,GLint,GLenum],
	doc='gluNurbsSurface( POINTER(GLUnurbs)(nurb), GLint(sKnotCount), arrays.GLfloatArray(sKnots), GLint(tKnotCount), arrays.GLfloatArray(tKnots), GLint(sStride), GLint(tStride), arrays.GLfloatArray(control), GLint(sOrder), GLint(tOrder), GLenum(type) ) -> None', 
	argNames=('nurb', 'sKnotCount', 'sKnots', 'tKnotCount', 'tKnots', 'sStride', 'tStride', 'control', 'sOrder', 'tOrder', 'type'),
)


# /usr/include/GL/glu.h 312
gluOrtho2D = platform.createBaseFunction( 
	'gluOrtho2D', dll=platform.GLU, resultType=None, 
	argTypes=[GLdouble,GLdouble,GLdouble,GLdouble],
	doc='gluOrtho2D( GLdouble(left), GLdouble(right), GLdouble(bottom), GLdouble(top) ) -> None', 
	argNames=('left', 'right', 'bottom', 'top'),
)


# /usr/include/GL/glu.h 313
gluPartialDisk = platform.createBaseFunction( 
	'gluPartialDisk', dll=platform.GLU, resultType=None, 
	argTypes=[POINTER(GLUquadric),GLdouble,GLdouble,GLint,GLint,GLdouble,GLdouble],
	doc='gluPartialDisk( POINTER(GLUquadric)(quad), GLdouble(inner), GLdouble(outer), GLint(slices), GLint(loops), GLdouble(start), GLdouble(sweep) ) -> None', 
	argNames=('quad', 'inner', 'outer', 'slices', 'loops', 'start', 'sweep'),
)


# /usr/include/GL/glu.h 314
gluPerspective = platform.createBaseFunction( 
	'gluPerspective', dll=platform.GLU, resultType=None, 
	argTypes=[GLdouble,GLdouble,GLdouble,GLdouble],
	doc='gluPerspective( GLdouble(fovy), GLdouble(aspect), GLdouble(zNear), GLdouble(zFar) ) -> None', 
	argNames=('fovy', 'aspect', 'zNear', 'zFar'),
)


# /usr/include/GL/glu.h 315
gluPickMatrix = platform.createBaseFunction( 
	'gluPickMatrix', dll=platform.GLU, resultType=None, 
	argTypes=[GLdouble,GLdouble,GLdouble,GLdouble,arrays.GLintArray],
	doc='gluPickMatrix( GLdouble(x), GLdouble(y), GLdouble(delX), GLdouble(delY), arrays.GLintArray(viewport) ) -> None', 
	argNames=('x', 'y', 'delX', 'delY', 'viewport'),
)


# /usr/include/GL/glu.h 316
gluProject = platform.createBaseFunction( 
	'gluProject', dll=platform.GLU, resultType=GLint, 
	argTypes=[GLdouble,GLdouble,GLdouble,arrays.GLdoubleArray,arrays.GLdoubleArray,arrays.GLintArray,arrays.GLdoubleArray,arrays.GLdoubleArray,arrays.GLdoubleArray],
	doc='gluProject( GLdouble(objX), GLdouble(objY), GLdouble(objZ), arrays.GLdoubleArray(model), arrays.GLdoubleArray(proj), arrays.GLintArray(view), arrays.GLdoubleArray(winX), arrays.GLdoubleArray(winY), arrays.GLdoubleArray(winZ) ) -> GLint', 
	argNames=('objX', 'objY', 'objZ', 'model', 'proj', 'view', 'winX', 'winY', 'winZ'),
)


# /usr/include/GL/glu.h 317
gluPwlCurve = platform.createBaseFunction( 
	'gluPwlCurve', dll=platform.GLU, resultType=None, 
	argTypes=[POINTER(GLUnurbs),GLint,arrays.GLfloatArray,GLint,GLenum],
	doc='gluPwlCurve( POINTER(GLUnurbs)(nurb), GLint(count), arrays.GLfloatArray(data), GLint(stride), GLenum(type) ) -> None', 
	argNames=('nurb', 'count', 'data', 'stride', 'type'),
)


# /usr/include/GL/glu.h 318
gluQuadricCallback = platform.createBaseFunction( 
	'gluQuadricCallback', dll=platform.GLU, resultType=None, 
	argTypes=[POINTER(GLUquadric),GLenum,_GLUfuncptr],
	doc='gluQuadricCallback( POINTER(GLUquadric)(quad), GLenum(which), _GLUfuncptr(CallBackFunc) ) -> None', 
	argNames=('quad', 'which', 'CallBackFunc'),
)


# /usr/include/GL/glu.h 319
gluQuadricDrawStyle = platform.createBaseFunction( 
	'gluQuadricDrawStyle', dll=platform.GLU, resultType=None, 
	argTypes=[POINTER(GLUquadric),GLenum],
	doc='gluQuadricDrawStyle( POINTER(GLUquadric)(quad), GLenum(draw) ) -> None', 
	argNames=('quad', 'draw'),
)


# /usr/include/GL/glu.h 320
gluQuadricNormals = platform.createBaseFunction( 
	'gluQuadricNormals', dll=platform.GLU, resultType=None, 
	argTypes=[POINTER(GLUquadric),GLenum],
	doc='gluQuadricNormals( POINTER(GLUquadric)(quad), GLenum(normal) ) -> None', 
	argNames=('quad', 'normal'),
)


# /usr/include/GL/glu.h 321
gluQuadricOrientation = platform.createBaseFunction( 
	'gluQuadricOrientation', dll=platform.GLU, resultType=None, 
	argTypes=[POINTER(GLUquadric),GLenum],
	doc='gluQuadricOrientation( POINTER(GLUquadric)(quad), GLenum(orientation) ) -> None', 
	argNames=('quad', 'orientation'),
)


# /usr/include/GL/glu.h 322
gluQuadricTexture = platform.createBaseFunction( 
	'gluQuadricTexture', dll=platform.GLU, resultType=None, 
	argTypes=[POINTER(GLUquadric),GLboolean],
	doc='gluQuadricTexture( POINTER(GLUquadric)(quad), GLboolean(texture) ) -> None', 
	argNames=('quad', 'texture'),
)


# /usr/include/GL/glu.h 323
gluScaleImage = platform.createBaseFunction( 
	'gluScaleImage', dll=platform.GLU, resultType=GLint, 
	argTypes=[GLenum,GLsizei,GLsizei,GLenum,c_void_p,GLsizei,GLsizei,GLenum,POINTER(GLvoid)],
	doc='gluScaleImage( GLenum(format), GLsizei(wIn), GLsizei(hIn), GLenum(typeIn), c_void_p(dataIn), GLsizei(wOut), GLsizei(hOut), GLenum(typeOut), POINTER(GLvoid)(dataOut) ) -> GLint', 
	argNames=('format', 'wIn', 'hIn', 'typeIn', 'dataIn', 'wOut', 'hOut', 'typeOut', 'dataOut'),
)


# /usr/include/GL/glu.h 324
gluSphere = platform.createBaseFunction( 
	'gluSphere', dll=platform.GLU, resultType=None, 
	argTypes=[POINTER(GLUquadric),GLdouble,GLint,GLint],
	doc='gluSphere( POINTER(GLUquadric)(quad), GLdouble(radius), GLint(slices), GLint(stacks) ) -> None', 
	argNames=('quad', 'radius', 'slices', 'stacks'),
)


# /usr/include/GL/glu.h 325
gluTessBeginContour = platform.createBaseFunction( 
	'gluTessBeginContour', dll=platform.GLU, resultType=None, 
	argTypes=[POINTER(GLUtesselator)],
	doc='gluTessBeginContour( POINTER(GLUtesselator)(tess) ) -> None', 
	argNames=('tess',),
)


# /usr/include/GL/glu.h 326
gluTessBeginPolygon = platform.createBaseFunction( 
	'gluTessBeginPolygon', dll=platform.GLU, resultType=None, 
	argTypes=[POINTER(GLUtesselator),POINTER(GLvoid)],
	doc='gluTessBeginPolygon( POINTER(GLUtesselator)(tess), POINTER(GLvoid)(data) ) -> None', 
	argNames=('tess', 'data'),
)


# /usr/include/GL/glu.h 327
gluTessCallback = platform.createBaseFunction( 
	'gluTessCallback', dll=platform.GLU, resultType=None, 
	argTypes=[POINTER(GLUtesselator),GLenum,_GLUfuncptr],
	doc='gluTessCallback( POINTER(GLUtesselator)(tess), GLenum(which), _GLUfuncptr(CallBackFunc) ) -> None', 
	argNames=('tess', 'which', 'CallBackFunc'),
)


# /usr/include/GL/glu.h 328
gluTessEndContour = platform.createBaseFunction( 
	'gluTessEndContour', dll=platform.GLU, resultType=None, 
	argTypes=[POINTER(GLUtesselator)],
	doc='gluTessEndContour( POINTER(GLUtesselator)(tess) ) -> None', 
	argNames=('tess',),
)


# /usr/include/GL/glu.h 329
gluTessEndPolygon = platform.createBaseFunction( 
	'gluTessEndPolygon', dll=platform.GLU, resultType=None, 
	argTypes=[POINTER(GLUtesselator)],
	doc='gluTessEndPolygon( POINTER(GLUtesselator)(tess) ) -> None', 
	argNames=('tess',),
)


# /usr/include/GL/glu.h 330
gluTessNormal = platform.createBaseFunction( 
	'gluTessNormal', dll=platform.GLU, resultType=None, 
	argTypes=[POINTER(GLUtesselator),GLdouble,GLdouble,GLdouble],
	doc='gluTessNormal( POINTER(GLUtesselator)(tess), GLdouble(valueX), GLdouble(valueY), GLdouble(valueZ) ) -> None', 
	argNames=('tess', 'valueX', 'valueY', 'valueZ'),
)


# /usr/include/GL/glu.h 331
gluTessProperty = platform.createBaseFunction( 
	'gluTessProperty', dll=platform.GLU, resultType=None, 
	argTypes=[POINTER(GLUtesselator),GLenum,GLdouble],
	doc='gluTessProperty( POINTER(GLUtesselator)(tess), GLenum(which), GLdouble(data) ) -> None', 
	argNames=('tess', 'which', 'data'),
)


# /usr/include/GL/glu.h 332
gluTessVertex = platform.createBaseFunction( 
	'gluTessVertex', dll=platform.GLU, resultType=None, 
	argTypes=[POINTER(GLUtesselator),arrays.GLdoubleArray,POINTER(GLvoid)],
	doc='gluTessVertex( POINTER(GLUtesselator)(tess), arrays.GLdoubleArray(location), POINTER(GLvoid)(data) ) -> None', 
	argNames=('tess', 'location', 'data'),
)


# /usr/include/GL/glu.h 333
gluUnProject = platform.createBaseFunction( 
	'gluUnProject', dll=platform.GLU, resultType=GLint, 
	argTypes=[GLdouble,GLdouble,GLdouble,arrays.GLdoubleArray,arrays.GLdoubleArray,arrays.GLintArray,arrays.GLdoubleArray,arrays.GLdoubleArray,arrays.GLdoubleArray],
	doc='gluUnProject( GLdouble(winX), GLdouble(winY), GLdouble(winZ), arrays.GLdoubleArray(model), arrays.GLdoubleArray(proj), arrays.GLintArray(view), arrays.GLdoubleArray(objX), arrays.GLdoubleArray(objY), arrays.GLdoubleArray(objZ) ) -> GLint', 
	argNames=('winX', 'winY', 'winZ', 'model', 'proj', 'view', 'objX', 'objY', 'objZ'),
)


# /usr/include/GL/glu.h 334
gluUnProject4 = platform.createBaseFunction( 
	'gluUnProject4', dll=platform.GLU, resultType=GLint, 
	argTypes=[GLdouble,GLdouble,GLdouble,GLdouble,arrays.GLdoubleArray,arrays.GLdoubleArray,arrays.GLintArray,GLdouble,GLdouble,arrays.GLdoubleArray,arrays.GLdoubleArray,arrays.GLdoubleArray,arrays.GLdoubleArray],
	doc='gluUnProject4( GLdouble(winX), GLdouble(winY), GLdouble(winZ), GLdouble(clipW), arrays.GLdoubleArray(model), arrays.GLdoubleArray(proj), arrays.GLintArray(view), GLdouble(nearVal), GLdouble(farVal), arrays.GLdoubleArray(objX), arrays.GLdoubleArray(objY), arrays.GLdoubleArray(objZ), arrays.GLdoubleArray(objW) ) -> GLint', 
	argNames=('winX', 'winY', 'winZ', 'clipW', 'model', 'proj', 'view', 'nearVal', 'farVal', 'objX', 'objY', 'objZ', 'objW'),
)

__all__ = [
	'GLU_AUTO_LOAD_MATRIX',
	'GLU_BEGIN',
	'GLU_CCW',
	'GLU_CULLING',
	'GLU_CW',
	'GLU_DISPLAY_MODE',
	'GLU_DOMAIN_DISTANCE',
	'GLU_EDGE_FLAG',
	'GLU_END',
	'GLU_ERROR',
	'GLU_EXTENSIONS',
	'GLU_EXTERIOR',
	'GLU_FALSE',
	'GLU_FILL',
	'GLU_FLAT',
	'GLU_INCOMPATIBLE_GL_VERSION',
	'GLU_INSIDE',
	'GLU_INTERIOR',
	'GLU_INVALID_ENUM',
	'GLU_INVALID_OPERATION',
	'GLU_INVALID_VALUE',
	'GLU_LINE',
	'GLU_MAP1_TRIM_2',
	'GLU_MAP1_TRIM_3',
	'GLU_NONE',
	'GLU_NURBS_BEGIN',
	'GLU_NURBS_BEGIN_DATA',
	'GLU_NURBS_BEGIN_DATA_EXT',
	'GLU_NURBS_BEGIN_EXT',
	'GLU_NURBS_COLOR',
	'GLU_NURBS_COLOR_DATA',
	'GLU_NURBS_COLOR_DATA_EXT',
	'GLU_NURBS_COLOR_EXT',
	'GLU_NURBS_END',
	'GLU_NURBS_END_DATA',
	'GLU_NURBS_END_DATA_EXT',
	'GLU_NURBS_END_EXT',
	'GLU_NURBS_ERROR',
	'GLU_NURBS_ERROR1',
	'GLU_NURBS_ERROR10',
	'GLU_NURBS_ERROR11',
	'GLU_NURBS_ERROR12',
	'GLU_NURBS_ERROR13',
	'GLU_NURBS_ERROR14',
	'GLU_NURBS_ERROR15',
	'GLU_NURBS_ERROR16',
	'GLU_NURBS_ERROR17',
	'GLU_NURBS_ERROR18',
	'GLU_NURBS_ERROR19',
	'GLU_NURBS_ERROR2',
	'GLU_NURBS_ERROR20',
	'GLU_NURBS_ERROR21',
	'GLU_NURBS_ERROR22',
	'GLU_NURBS_ERROR23',
	'GLU_NURBS_ERROR24',
	'GLU_NURBS_ERROR25',
	'GLU_NURBS_ERROR26',
	'GLU_NURBS_ERROR27',
	'GLU_NURBS_ERROR28',
	'GLU_NURBS_ERROR29',
	'GLU_NURBS_ERROR3',
	'GLU_NURBS_ERROR30',
	'GLU_NURBS_ERROR31',
	'GLU_NURBS_ERROR32',
	'GLU_NURBS_ERROR33',
	'GLU_NURBS_ERROR34',
	'GLU_NURBS_ERROR35',
	'GLU_NURBS_ERROR36',
	'GLU_NURBS_ERROR37',
	'GLU_NURBS_ERROR4',
	'GLU_NURBS_ERROR5',
	'GLU_NURBS_ERROR6',
	'GLU_NURBS_ERROR7',
	'GLU_NURBS_ERROR8',
	'GLU_NURBS_ERROR9',
	'GLU_NURBS_MODE',
	'GLU_NURBS_MODE_EXT',
	'GLU_NURBS_NORMAL',
	'GLU_NURBS_NORMAL_DATA',
	'GLU_NURBS_NORMAL_DATA_EXT',
	'GLU_NURBS_NORMAL_EXT',
	'GLU_NURBS_RENDERER',
	'GLU_NURBS_RENDERER_EXT',
	'GLU_NURBS_TESSELLATOR',
	'GLU_NURBS_TESSELLATOR_EXT',
	'GLU_NURBS_TEXTURE_COORD',
	'GLU_NURBS_TEXTURE_COORD_DATA',
	'GLU_NURBS_TEX_COORD_DATA_EXT',
	'GLU_NURBS_TEX_COORD_EXT',
	'GLU_NURBS_VERTEX',
	'GLU_NURBS_VERTEX_DATA',
	'GLU_NURBS_VERTEX_DATA_EXT',
	'GLU_NURBS_VERTEX_EXT',
	'GLU_OBJECT_PARAMETRIC_ERROR',
	'GLU_OBJECT_PARAMETRIC_ERROR_EXT',
	'GLU_OBJECT_PATH_LENGTH',
	'GLU_OBJECT_PATH_LENGTH_EXT',
	'GLU_OUTLINE_PATCH',
	'GLU_OUTLINE_POLYGON',
	'GLU_OUTSIDE',
	'GLU_OUT_OF_MEMORY',
	'GLU_PARAMETRIC_ERROR',
	'GLU_PARAMETRIC_TOLERANCE',
	'GLU_PATH_LENGTH',
	'GLU_POINT',
	'GLU_SAMPLING_METHOD',
	'GLU_SAMPLING_TOLERANCE',
	'GLU_SILHOUETTE',
	'GLU_SMOOTH',
	'GLU_TESS_BEGIN',
	'GLU_TESS_BEGIN_DATA',
	'GLU_TESS_BOUNDARY_ONLY',
	'GLU_TESS_COMBINE',
	'GLU_TESS_COMBINE_DATA',
	'GLU_TESS_COORD_TOO_LARGE',
	'GLU_TESS_EDGE_FLAG',
	'GLU_TESS_EDGE_FLAG_DATA',
	'GLU_TESS_END',
	'GLU_TESS_END_DATA',
	'GLU_TESS_ERROR',
	'GLU_TESS_ERROR1',
	'GLU_TESS_ERROR2',
	'GLU_TESS_ERROR3',
	'GLU_TESS_ERROR4',
	'GLU_TESS_ERROR5',
	'GLU_TESS_ERROR6',
	'GLU_TESS_ERROR7',
	'GLU_TESS_ERROR8',
	'GLU_TESS_ERROR_DATA',
	'GLU_TESS_MAX_COORD',
	'GLU_TESS_MISSING_BEGIN_CONTOUR',
	'GLU_TESS_MISSING_BEGIN_POLYGON',
	'GLU_TESS_MISSING_END_CONTOUR',
	'GLU_TESS_MISSING_END_POLYGON',
	'GLU_TESS_NEED_COMBINE_CALLBACK',
	'GLU_TESS_TOLERANCE',
	'GLU_TESS_VERTEX',
	'GLU_TESS_VERTEX_DATA',
	'GLU_TESS_WINDING_ABS_GEQ_TWO',
	'GLU_TESS_WINDING_NEGATIVE',
	'GLU_TESS_WINDING_NONZERO',
	'GLU_TESS_WINDING_ODD',
	'GLU_TESS_WINDING_POSITIVE',
	'GLU_TESS_WINDING_RULE',
	'GLU_TRUE',
	'GLU_UNKNOWN',
	'GLU_U_STEP',
	'GLU_VERSION',
	'GLU_VERSION_1_1',
	'GLU_VERSION_1_2',
	'GLU_VERSION_1_3',
	'GLU_VERTEX',
	'GLU_V_STEP',
	'GLUnurbs',
	'GLUnurbsObj',
	'GLUquadric',
	'GLUquadricObj',
	'GLUtesselator',
	'GLUtesselatorObj',
	'GLUtriangulatorObj',
	'GLboolean',
	'GLdouble',
	'GLenum',
	'GLfloat',
	'GLint',
	'GLsizei',
	'GLubyte',
	'GLvoid',
	'_GLUfuncptr',
	'gluBeginCurve',
	'gluBeginPolygon',
	'gluBeginSurface',
	'gluBeginTrim',
	'gluBuild1DMipmapLevels',
	'gluBuild1DMipmaps',
	'gluBuild2DMipmapLevels',
	'gluBuild2DMipmaps',
	'gluBuild3DMipmapLevels',
	'gluBuild3DMipmaps',
	'gluCheckExtension',
	'gluCylinder',
	'gluDeleteNurbsRenderer',
	'gluDeleteQuadric',
	'gluDeleteTess',
	'gluDisk',
	'gluEndCurve',
	'gluEndPolygon',
	'gluEndSurface',
	'gluEndTrim',
	'gluErrorString',
	'gluGetNurbsProperty',
	'gluGetString',
	'gluGetTessProperty',
	'gluLoadSamplingMatrices',
	'gluLookAt',
	'gluNewNurbsRenderer',
	'gluNewQuadric',
	'gluNewTess',
	'gluNextContour',
	'gluNurbsCallback',
	'gluNurbsCallbackData',
	'gluNurbsCallbackDataEXT',
	'gluNurbsCurve',
	'gluNurbsProperty',
	'gluNurbsSurface',
	'gluOrtho2D',
	'gluPartialDisk',
	'gluPerspective',
	'gluPickMatrix',
	'gluProject',
	'gluPwlCurve',
	'gluQuadricCallback',
	'gluQuadricDrawStyle',
	'gluQuadricNormals',
	'gluQuadricOrientation',
	'gluQuadricTexture',
	'gluScaleImage',
	'gluSphere',
	'gluTessBeginContour',
	'gluTessBeginPolygon',
	'gluTessCallback',
	'gluTessEndContour',
	'gluTessEndPolygon',
	'gluTessNormal',
	'gluTessProperty',
	'gluTessVertex',
	'gluUnProject',
	'gluUnProject4'
]




'''OpenGL extension EXT.draw_range_elements

This module customises the behaviour of the 
OpenGL.raw.GL.EXT.draw_range_elements to provide a more 
Python-friendly API
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions, wrapper
from OpenGL.GL import glget
import ctypes
from OpenGL.raw.GL.EXT.draw_range_elements import *
### END AUTOGENERATED SECTION

glDrawRangeElementsEXT = wrapper.wrapper( glDrawRangeElementsEXT ).setPyConverter(
	'indices', arrays.AsArrayOfType( 'indices', 'type' ),
).setCResolver( 
	'indices', arrays.ArrayDatatype.voidDataPointer ,
).setReturnValues( 
	wrapper.returnPyArgument( 'indices' ) 
)



"""Numbers passed as array handling code for PyOpenGL
"""
REGISTRY_NAME = 'numbers'
from OpenGL import constants
from OpenGL.arrays import formathandler
import ctypes

class NumberHandler( formathandler.FormatHandler ):
	"""Allows the user to pass a bald Python float,int, etceteras as an array-of-1"""
	HANDLED_TYPES = (
		int,long,float,
		constants.GLdouble,
		constants.GLfloat,
		constants.GLint,
		constants.GLshort,
		constants.GLuint,
		constants.GLushort, 
		constants.GLclampf,
		constants.GLclampd,
	)
	def from_param( self, value ):
		"""If it's a ctypes value, pass on, otherwise do asArray"""
		if not isinstance( value, TARGET_TYPE_TUPLE ):
			return self.asArray( value )
		return ctypes.byref(value)
	dataPointer = from_param
	def zeros( self, dims, typeCode=None ):
		"""Currently don't allow Number as output types!"""
		raise NotImplemented( """Number data-type not allowed as an output array format""" )
	def ones( self, dims, typeCode=None ):
		"""Currently don't allow Number as output types!"""
		raise NotImplemented( """Number data-type not allowed as an output array format""" )
	def arrayToGLType( self, value ):
		"""Given a value, guess OpenGL type of the corresponding pointer"""
		if isinstance( value, TARGET_TYPE_TUPLE ):
			return TARGET_TYPES[ type(value) ]
		else:
			guess = DEFAULT_TYPES.get( type(value))
			if guess is not None:
				return guess[1]
			raise TypeError( """Can't guess array data-type for %r types"""%(type(value)))
	def arraySize( self, value, typeCode = None ):
		"""Given a data-value, calculate ravelled size for the array"""
		return 1
	def asArray( self, value, typeCode=None ):
		"""Convert given value to an array value of given typeCode"""
		if isinstance( value, TARGET_TYPE_TUPLE ):
			return value
		targetType = CONSTANT_TO_TYPE.get( typeCode )
		if targetType is not None:
			return targetType( value )
		raise TypeError( """Don't know how to convert %r to an array type"""%(
			typeCode,
		))
	def unitSize( self, value, typeCode=None ):
		"""Determine unit size of an array (if possible)"""
		return 1 # there's only 1 possible value in the set...
	def registerEquivalent( self, typ, base ):
		"""Register a sub-class for handling as the base-type"""
		global TARGET_TYPE_TUPLE
		for source in (DEFAULT_TYPES, TARGET_TYPES, BYTE_SIZES):
			if source.has_key( base ):
				source[typ] = source[base]
		if TARGET_TYPES.has_key( base ):
			TARGET_TYPE_TUPLE = TARGET_TYPE_TUPLE + (base,)

DEFAULT_TYPES = {
	float: (constants.GLdouble,constants.GL_DOUBLE),
	int: (constants.GLint,constants.GL_INT),
	long: (constants.GLint,constants.GL_INT),
}
TARGET_TYPES = dict([
	(getattr( constants,n),c)
	for (n,c) in constants.ARRAY_TYPE_TO_CONSTANT
])
TARGET_TYPE_TUPLE = tuple([
	getattr(constants,n) 
	for (n,c) in constants.ARRAY_TYPE_TO_CONSTANT
])
CONSTANT_TO_TYPE = dict([
	(c,getattr( constants, n))
	for (n,c) in constants.ARRAY_TYPE_TO_CONSTANT
])

BYTE_SIZES = dict([
	( c, ctypes.sizeof( getattr( constants, n) ) )
	for (n,c) in constants.ARRAY_TYPE_TO_CONSTANT
])
del n,c



'''OpenGL extension VERSION.GL_2_0

The official definition of this extension is available here:
	http://oss.sgi.com/projects/ogl-sample/registry/VERSION/GL_2_0.txt

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_VERSION_GL_2_0'
GL_VERTEX_ATTRIB_ARRAY_ENABLED = constant.Constant( 'GL_VERTEX_ATTRIB_ARRAY_ENABLED', 0x8622 )
GL_VERTEX_ATTRIB_ARRAY_SIZE = constant.Constant( 'GL_VERTEX_ATTRIB_ARRAY_SIZE', 0x8623 )
GL_VERTEX_ATTRIB_ARRAY_STRIDE = constant.Constant( 'GL_VERTEX_ATTRIB_ARRAY_STRIDE', 0x8624 )
GL_VERTEX_ATTRIB_ARRAY_TYPE = constant.Constant( 'GL_VERTEX_ATTRIB_ARRAY_TYPE', 0x8625 )
GL_CURRENT_VERTEX_ATTRIB = constant.Constant( 'GL_CURRENT_VERTEX_ATTRIB', 0x8626 )
GL_VERTEX_PROGRAM_POINT_SIZE = constant.Constant( 'GL_VERTEX_PROGRAM_POINT_SIZE', 0x8642 )
GL_VERTEX_PROGRAM_TWO_SIDE = constant.Constant( 'GL_VERTEX_PROGRAM_TWO_SIDE', 0x8643 )
GL_VERTEX_ATTRIB_ARRAY_POINTER = constant.Constant( 'GL_VERTEX_ATTRIB_ARRAY_POINTER', 0x8645 )
GL_STENCIL_BACK_FUNC = constant.Constant( 'GL_STENCIL_BACK_FUNC', 0x8800 )
GL_STENCIL_BACK_FAIL = constant.Constant( 'GL_STENCIL_BACK_FAIL', 0x8801 )
GL_STENCIL_BACK_PASS_DEPTH_FAIL = constant.Constant( 'GL_STENCIL_BACK_PASS_DEPTH_FAIL', 0x8802 )
GL_STENCIL_BACK_PASS_DEPTH_PASS = constant.Constant( 'GL_STENCIL_BACK_PASS_DEPTH_PASS', 0x8803 )
GL_MAX_DRAW_BUFFERS = constant.Constant( 'GL_MAX_DRAW_BUFFERS', 0x8824 )
GL_DRAW_BUFFER0 = constant.Constant( 'GL_DRAW_BUFFER0', 0x8825 )
GL_DRAW_BUFFER1 = constant.Constant( 'GL_DRAW_BUFFER1', 0x8826 )
GL_DRAW_BUFFER2 = constant.Constant( 'GL_DRAW_BUFFER2', 0x8827 )
GL_DRAW_BUFFER3 = constant.Constant( 'GL_DRAW_BUFFER3', 0x8828 )
GL_DRAW_BUFFER4 = constant.Constant( 'GL_DRAW_BUFFER4', 0x8829 )
GL_DRAW_BUFFER5 = constant.Constant( 'GL_DRAW_BUFFER5', 0x882A )
GL_DRAW_BUFFER6 = constant.Constant( 'GL_DRAW_BUFFER6', 0x882B )
GL_DRAW_BUFFER7 = constant.Constant( 'GL_DRAW_BUFFER7', 0x882C )
GL_DRAW_BUFFER8 = constant.Constant( 'GL_DRAW_BUFFER8', 0x882D )
GL_DRAW_BUFFER9 = constant.Constant( 'GL_DRAW_BUFFER9', 0x882E )
GL_DRAW_BUFFER10 = constant.Constant( 'GL_DRAW_BUFFER10', 0x882F )
GL_DRAW_BUFFER11 = constant.Constant( 'GL_DRAW_BUFFER11', 0x8830 )
GL_DRAW_BUFFER12 = constant.Constant( 'GL_DRAW_BUFFER12', 0x8831 )
GL_DRAW_BUFFER13 = constant.Constant( 'GL_DRAW_BUFFER13', 0x8832 )
GL_DRAW_BUFFER14 = constant.Constant( 'GL_DRAW_BUFFER14', 0x8833 )
GL_DRAW_BUFFER15 = constant.Constant( 'GL_DRAW_BUFFER15', 0x8834 )
GL_BLEND_EQUATION_ALPHA = constant.Constant( 'GL_BLEND_EQUATION_ALPHA', 0x883D )
GL_POINT_SPRITE = constant.Constant( 'GL_POINT_SPRITE', 0x8861 )
GL_COORD_REPLACE = constant.Constant( 'GL_COORD_REPLACE', 0x8862 )
GL_MAX_VERTEX_ATTRIBS = constant.Constant( 'GL_MAX_VERTEX_ATTRIBS', 0x8869 )
GL_VERTEX_ATTRIB_ARRAY_NORMALIZED = constant.Constant( 'GL_VERTEX_ATTRIB_ARRAY_NORMALIZED', 0x886A )
GL_MAX_TEXTURE_COORDS = constant.Constant( 'GL_MAX_TEXTURE_COORDS', 0x8871 )
GL_MAX_TEXTURE_IMAGE_UNITS = constant.Constant( 'GL_MAX_TEXTURE_IMAGE_UNITS', 0x8872 )
GL_FRAGMENT_SHADER = constant.Constant( 'GL_FRAGMENT_SHADER', 0x8B30 )
GL_VERTEX_SHADER = constant.Constant( 'GL_VERTEX_SHADER', 0x8B31 )
GL_MAX_FRAGMENT_UNIFORM_COMPONENTS = constant.Constant( 'GL_MAX_FRAGMENT_UNIFORM_COMPONENTS', 0x8B49 )
GL_MAX_VERTEX_UNIFORM_COMPONENTS = constant.Constant( 'GL_MAX_VERTEX_UNIFORM_COMPONENTS', 0x8B4A )
GL_MAX_VARYING_FLOATS = constant.Constant( 'GL_MAX_VARYING_FLOATS', 0x8B4B )
GL_MAX_VERTEX_TEXTURE_IMAGE_UNITS = constant.Constant( 'GL_MAX_VERTEX_TEXTURE_IMAGE_UNITS', 0x8B4C )
GL_MAX_COMBINED_TEXTURE_IMAGE_UNITS = constant.Constant( 'GL_MAX_COMBINED_TEXTURE_IMAGE_UNITS', 0x8B4D )
GL_SHADER_TYPE = constant.Constant( 'GL_SHADER_TYPE', 0x8B4F )
GL_FLOAT_VEC2 = constant.Constant( 'GL_FLOAT_VEC2', 0x8B50 )
GL_FLOAT_VEC3 = constant.Constant( 'GL_FLOAT_VEC3', 0x8B51 )
GL_FLOAT_VEC4 = constant.Constant( 'GL_FLOAT_VEC4', 0x8B52 )
GL_INT_VEC2 = constant.Constant( 'GL_INT_VEC2', 0x8B53 )
GL_INT_VEC3 = constant.Constant( 'GL_INT_VEC3', 0x8B54 )
GL_INT_VEC4 = constant.Constant( 'GL_INT_VEC4', 0x8B55 )
GL_BOOL = constant.Constant( 'GL_BOOL', 0x8B56 )
GL_BOOL_VEC2 = constant.Constant( 'GL_BOOL_VEC2', 0x8B57 )
GL_BOOL_VEC3 = constant.Constant( 'GL_BOOL_VEC3', 0x8B58 )
GL_BOOL_VEC4 = constant.Constant( 'GL_BOOL_VEC4', 0x8B59 )
GL_FLOAT_MAT2 = constant.Constant( 'GL_FLOAT_MAT2', 0x8B5A )
GL_FLOAT_MAT3 = constant.Constant( 'GL_FLOAT_MAT3', 0x8B5B )
GL_FLOAT_MAT4 = constant.Constant( 'GL_FLOAT_MAT4', 0x8B5C )
GL_SAMPLER_1D = constant.Constant( 'GL_SAMPLER_1D', 0x8B5D )
GL_SAMPLER_2D = constant.Constant( 'GL_SAMPLER_2D', 0x8B5E )
GL_SAMPLER_3D = constant.Constant( 'GL_SAMPLER_3D', 0x8B5F )
GL_SAMPLER_CUBE = constant.Constant( 'GL_SAMPLER_CUBE', 0x8B60 )
GL_SAMPLER_1D_SHADOW = constant.Constant( 'GL_SAMPLER_1D_SHADOW', 0x8B61 )
GL_SAMPLER_2D_SHADOW = constant.Constant( 'GL_SAMPLER_2D_SHADOW', 0x8B62 )
GL_DELETE_STATUS = constant.Constant( 'GL_DELETE_STATUS', 0x8B80 )
GL_COMPILE_STATUS = constant.Constant( 'GL_COMPILE_STATUS', 0x8B81 )
GL_LINK_STATUS = constant.Constant( 'GL_LINK_STATUS', 0x8B82 )
GL_VALIDATE_STATUS = constant.Constant( 'GL_VALIDATE_STATUS', 0x8B83 )
GL_INFO_LOG_LENGTH = constant.Constant( 'GL_INFO_LOG_LENGTH', 0x8B84 )
GL_ATTACHED_SHADERS = constant.Constant( 'GL_ATTACHED_SHADERS', 0x8B85 )
GL_ACTIVE_UNIFORMS = constant.Constant( 'GL_ACTIVE_UNIFORMS', 0x8B86 )
GL_ACTIVE_UNIFORM_MAX_LENGTH = constant.Constant( 'GL_ACTIVE_UNIFORM_MAX_LENGTH', 0x8B87 )
GL_SHADER_SOURCE_LENGTH = constant.Constant( 'GL_SHADER_SOURCE_LENGTH', 0x8B88 )
GL_ACTIVE_ATTRIBUTES = constant.Constant( 'GL_ACTIVE_ATTRIBUTES', 0x8B89 )
GL_ACTIVE_ATTRIBUTE_MAX_LENGTH = constant.Constant( 'GL_ACTIVE_ATTRIBUTE_MAX_LENGTH', 0x8B8A )
GL_FRAGMENT_SHADER_DERIVATIVE_HINT = constant.Constant( 'GL_FRAGMENT_SHADER_DERIVATIVE_HINT', 0x8B8B )
GL_SHADING_LANGUAGE_VERSION = constant.Constant( 'GL_SHADING_LANGUAGE_VERSION', 0x8B8C )
GL_CURRENT_PROGRAM = constant.Constant( 'GL_CURRENT_PROGRAM', 0x8B8D )
GL_POINT_SPRITE_COORD_ORIGIN = constant.Constant( 'GL_POINT_SPRITE_COORD_ORIGIN', 0x8CA0 )
GL_LOWER_LEFT = constant.Constant( 'GL_LOWER_LEFT', 0x8CA1 )
GL_UPPER_LEFT = constant.Constant( 'GL_UPPER_LEFT', 0x8CA2 )
GL_STENCIL_BACK_REF = constant.Constant( 'GL_STENCIL_BACK_REF', 0x8CA3 )
GL_STENCIL_BACK_VALUE_MASK = constant.Constant( 'GL_STENCIL_BACK_VALUE_MASK', 0x8CA4 )
GL_STENCIL_BACK_WRITEMASK = constant.Constant( 'GL_STENCIL_BACK_WRITEMASK', 0x8CA5 )
glBlendEquationSeparate = platform.createExtensionFunction( 
	'glBlendEquationSeparate', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, constants.GLenum,),
	doc = 'glBlendEquationSeparate( GLenum(modeRGB), GLenum(modeAlpha) ) -> None',
	argNames = ('modeRGB', 'modeAlpha',),
)

glDrawBuffers = platform.createExtensionFunction( 
	'glDrawBuffers', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLsizei, arrays.GLuintArray,),
	doc = 'glDrawBuffers( GLsizei(n), GLuintArray(bufs) ) -> None',
	argNames = ('n', 'bufs',),
)

glStencilOpSeparate = platform.createExtensionFunction( 
	'glStencilOpSeparate', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, constants.GLenum, constants.GLenum, constants.GLenum,),
	doc = 'glStencilOpSeparate( GLenum(face), GLenum(sfail), GLenum(dpfail), GLenum(dppass) ) -> None',
	argNames = ('face', 'sfail', 'dpfail', 'dppass',),
)

glStencilFuncSeparate = platform.createExtensionFunction( 
	'glStencilFuncSeparate', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, constants.GLenum, constants.GLint, constants.GLuint,),
	doc = 'glStencilFuncSeparate( GLenum(frontfunc), GLenum(backfunc), GLint(ref), GLuint(mask) ) -> None',
	argNames = ('frontfunc', 'backfunc', 'ref', 'mask',),
)

glStencilMaskSeparate = platform.createExtensionFunction( 
	'glStencilMaskSeparate', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, constants.GLuint,),
	doc = 'glStencilMaskSeparate( GLenum(face), GLuint(mask) ) -> None',
	argNames = ('face', 'mask',),
)

glAttachShader = platform.createExtensionFunction( 
	'glAttachShader', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLuint, constants.GLuint,),
	doc = 'glAttachShader( GLuint(program), GLuint(shader) ) -> None',
	argNames = ('program', 'shader',),
)

glBindAttribLocation = platform.createExtensionFunction( 
	'glBindAttribLocation', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLuint, constants.GLuint, arrays.GLcharArray,),
	doc = 'glBindAttribLocation( GLuint(program), GLuint(index), GLcharArray(name) ) -> None',
	argNames = ('program', 'index', 'name',),
)

glCompileShader = platform.createExtensionFunction( 
	'glCompileShader', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLuint,),
	doc = 'glCompileShader( GLuint(shader) ) -> None',
	argNames = ('shader',),
)

glCreateProgram = platform.createExtensionFunction( 
	'glCreateProgram', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=constants.GLuint, 
	argTypes=(),
	doc = 'glCreateProgram(  ) -> constants.GLuint',
	argNames = (),
)

glCreateShader = platform.createExtensionFunction( 
	'glCreateShader', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=constants.GLuint, 
	argTypes=(constants.GLenum,),
	doc = 'glCreateShader( GLenum(type) ) -> constants.GLuint',
	argNames = ('type',),
)

glDeleteProgram = platform.createExtensionFunction( 
	'glDeleteProgram', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLuint,),
	doc = 'glDeleteProgram( GLuint(program) ) -> None',
	argNames = ('program',),
)

glDeleteShader = platform.createExtensionFunction( 
	'glDeleteShader', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLuint,),
	doc = 'glDeleteShader( GLuint(shader) ) -> None',
	argNames = ('shader',),
)

glDetachShader = platform.createExtensionFunction( 
	'glDetachShader', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLuint, constants.GLuint,),
	doc = 'glDetachShader( GLuint(program), GLuint(shader) ) -> None',
	argNames = ('program', 'shader',),
)

glDisableVertexAttribArray = platform.createExtensionFunction( 
	'glDisableVertexAttribArray', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLuint,),
	doc = 'glDisableVertexAttribArray( GLuint(index) ) -> None',
	argNames = ('index',),
)

glEnableVertexAttribArray = platform.createExtensionFunction( 
	'glEnableVertexAttribArray', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLuint,),
	doc = 'glEnableVertexAttribArray( GLuint(index) ) -> None',
	argNames = ('index',),
)

glGetActiveAttrib = platform.createExtensionFunction( 
	'glGetActiveAttrib', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLuint, constants.GLuint, constants.GLsizei, arrays.GLsizeiArray, arrays.GLintArray, arrays.GLuintArray, arrays.GLcharArray,),
	doc = 'glGetActiveAttrib( GLuint(program), GLuint(index), GLsizei(bufSize), GLsizeiArray(length), GLintArray(size), GLuintArray(type), GLcharArray(name) ) -> None',
	argNames = ('program', 'index', 'bufSize', 'length', 'size', 'type', 'name',),
)

glGetActiveUniform = platform.createExtensionFunction( 
	'glGetActiveUniform', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLuint, constants.GLuint, constants.GLsizei, arrays.GLsizeiArray, arrays.GLintArray, arrays.GLuintArray, arrays.GLcharArray,),
	doc = 'glGetActiveUniform( GLuint(program), GLuint(index), GLsizei(bufSize), GLsizeiArray(length), GLintArray(size), GLuintArray(type), GLcharArray(name) ) -> None',
	argNames = ('program', 'index', 'bufSize', 'length', 'size', 'type', 'name',),
)

glGetAttachedShaders = platform.createExtensionFunction( 
	'glGetAttachedShaders', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLuint, constants.GLsizei, arrays.GLsizeiArray, arrays.GLuintArray,),
	doc = 'glGetAttachedShaders( GLuint(program), GLsizei(maxCount), GLsizeiArray(count), GLuintArray(obj) ) -> None',
	argNames = ('program', 'maxCount', 'count', 'obj',),
)

glGetAttribLocation = platform.createExtensionFunction( 
	'glGetAttribLocation', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=constants.GLint, 
	argTypes=(constants.GLuint, arrays.GLcharArray,),
	doc = 'glGetAttribLocation( GLuint(program), GLcharArray(name) ) -> constants.GLint',
	argNames = ('program', 'name',),
)

glGetProgramiv = platform.createExtensionFunction( 
	'glGetProgramiv', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLuint, constants.GLenum, arrays.GLintArray,),
	doc = 'glGetProgramiv( GLuint(program), GLenum(pname), GLintArray(params) ) -> None',
	argNames = ('program', 'pname', 'params',),
)

glGetProgramInfoLog = platform.createExtensionFunction( 
	'glGetProgramInfoLog', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLuint, constants.GLsizei, arrays.GLsizeiArray, arrays.GLcharArray,),
	doc = 'glGetProgramInfoLog( GLuint(program), GLsizei(bufSize), GLsizeiArray(length), GLcharArray(infoLog) ) -> None',
	argNames = ('program', 'bufSize', 'length', 'infoLog',),
)

glGetShaderiv = platform.createExtensionFunction( 
	'glGetShaderiv', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLuint, constants.GLenum, arrays.GLintArray,),
	doc = 'glGetShaderiv( GLuint(shader), GLenum(pname), GLintArray(params) ) -> None',
	argNames = ('shader', 'pname', 'params',),
)

glGetShaderInfoLog = platform.createExtensionFunction( 
	'glGetShaderInfoLog', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLuint, constants.GLsizei, arrays.GLsizeiArray, arrays.GLcharArray,),
	doc = 'glGetShaderInfoLog( GLuint(shader), GLsizei(bufSize), GLsizeiArray(length), GLcharArray(infoLog) ) -> None',
	argNames = ('shader', 'bufSize', 'length', 'infoLog',),
)

glGetShaderSource = platform.createExtensionFunction( 
	'glGetShaderSource', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLuint, constants.GLsizei, arrays.GLsizeiArray, arrays.GLcharArray,),
	doc = 'glGetShaderSource( GLuint(shader), GLsizei(bufSize), GLsizeiArray(length), GLcharArray(source) ) -> None',
	argNames = ('shader', 'bufSize', 'length', 'source',),
)

glGetUniformLocation = platform.createExtensionFunction( 
	'glGetUniformLocation', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=constants.GLint, 
	argTypes=(constants.GLuint, arrays.GLcharArray,),
	doc = 'glGetUniformLocation( GLuint(program), GLcharArray(name) ) -> constants.GLint',
	argNames = ('program', 'name',),
)

glGetUniformfv = platform.createExtensionFunction( 
	'glGetUniformfv', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLuint, constants.GLint, arrays.GLfloatArray,),
	doc = 'glGetUniformfv( GLuint(program), GLint(location), GLfloatArray(params) ) -> None',
	argNames = ('program', 'location', 'params',),
)

glGetUniformiv = platform.createExtensionFunction( 
	'glGetUniformiv', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLuint, constants.GLint, arrays.GLintArray,),
	doc = 'glGetUniformiv( GLuint(program), GLint(location), GLintArray(params) ) -> None',
	argNames = ('program', 'location', 'params',),
)

glGetVertexAttribdv = platform.createExtensionFunction( 
	'glGetVertexAttribdv', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLuint, constants.GLenum, arrays.GLdoubleArray,),
	doc = 'glGetVertexAttribdv( GLuint(index), GLenum(pname), GLdoubleArray(params) ) -> None',
	argNames = ('index', 'pname', 'params',),
)

glGetVertexAttribfv = platform.createExtensionFunction( 
	'glGetVertexAttribfv', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLuint, constants.GLenum, arrays.GLfloatArray,),
	doc = 'glGetVertexAttribfv( GLuint(index), GLenum(pname), GLfloatArray(params) ) -> None',
	argNames = ('index', 'pname', 'params',),
)

glGetVertexAttribiv = platform.createExtensionFunction( 
	'glGetVertexAttribiv', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLuint, constants.GLenum, arrays.GLintArray,),
	doc = 'glGetVertexAttribiv( GLuint(index), GLenum(pname), GLintArray(params) ) -> None',
	argNames = ('index', 'pname', 'params',),
)

glGetVertexAttribPointerv = platform.createExtensionFunction( 
	'glGetVertexAttribPointerv', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLuint, constants.GLenum, ctypes.POINTER(ctypes.c_void_p),),
	doc = 'glGetVertexAttribPointerv( GLuint(index), GLenum(pname), POINTER(ctypes.c_void_p)(pointer) ) -> None',
	argNames = ('index', 'pname', 'pointer',),
)

glIsProgram = platform.createExtensionFunction( 
	'glIsProgram', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=constants.GLboolean, 
	argTypes=(constants.GLuint,),
	doc = 'glIsProgram( GLuint(program) ) -> constants.GLboolean',
	argNames = ('program',),
)

glIsShader = platform.createExtensionFunction( 
	'glIsShader', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=constants.GLboolean, 
	argTypes=(constants.GLuint,),
	doc = 'glIsShader( GLuint(shader) ) -> constants.GLboolean',
	argNames = ('shader',),
)

glLinkProgram = platform.createExtensionFunction( 
	'glLinkProgram', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLuint,),
	doc = 'glLinkProgram( GLuint(program) ) -> None',
	argNames = ('program',),
)

glShaderSource = platform.createExtensionFunction( 
	'glShaderSource', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLuint, constants.GLsizei, ctypes.POINTER(arrays.GLcharArray), arrays.GLintArray,),
	doc = 'glShaderSource( GLuint(shader), GLsizei(count), POINTER(arrays.GLcharArray)(string), GLintArray(length) ) -> None',
	argNames = ('shader', 'count', 'string', 'length',),
)

glUseProgram = platform.createExtensionFunction( 
	'glUseProgram', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLuint,),
	doc = 'glUseProgram( GLuint(program) ) -> None',
	argNames = ('program',),
)

glUniform1f = platform.createExtensionFunction( 
	'glUniform1f', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLint, constants.GLfloat,),
	doc = 'glUniform1f( GLint(location), GLfloat(v0) ) -> None',
	argNames = ('location', 'v0',),
)

glUniform2f = platform.createExtensionFunction( 
	'glUniform2f', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLint, constants.GLfloat, constants.GLfloat,),
	doc = 'glUniform2f( GLint(location), GLfloat(v0), GLfloat(v1) ) -> None',
	argNames = ('location', 'v0', 'v1',),
)

glUniform3f = platform.createExtensionFunction( 
	'glUniform3f', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLint, constants.GLfloat, constants.GLfloat, constants.GLfloat,),
	doc = 'glUniform3f( GLint(location), GLfloat(v0), GLfloat(v1), GLfloat(v2) ) -> None',
	argNames = ('location', 'v0', 'v1', 'v2',),
)

glUniform4f = platform.createExtensionFunction( 
	'glUniform4f', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLint, constants.GLfloat, constants.GLfloat, constants.GLfloat, constants.GLfloat,),
	doc = 'glUniform4f( GLint(location), GLfloat(v0), GLfloat(v1), GLfloat(v2), GLfloat(v3) ) -> None',
	argNames = ('location', 'v0', 'v1', 'v2', 'v3',),
)

glUniform1i = platform.createExtensionFunction( 
	'glUniform1i', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLint, constants.GLint,),
	doc = 'glUniform1i( GLint(location), GLint(v0) ) -> None',
	argNames = ('location', 'v0',),
)

glUniform2i = platform.createExtensionFunction( 
	'glUniform2i', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLint, constants.GLint, constants.GLint,),
	doc = 'glUniform2i( GLint(location), GLint(v0), GLint(v1) ) -> None',
	argNames = ('location', 'v0', 'v1',),
)

glUniform3i = platform.createExtensionFunction( 
	'glUniform3i', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLint, constants.GLint, constants.GLint, constants.GLint,),
	doc = 'glUniform3i( GLint(location), GLint(v0), GLint(v1), GLint(v2) ) -> None',
	argNames = ('location', 'v0', 'v1', 'v2',),
)

glUniform4i = platform.createExtensionFunction( 
	'glUniform4i', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLint, constants.GLint, constants.GLint, constants.GLint, constants.GLint,),
	doc = 'glUniform4i( GLint(location), GLint(v0), GLint(v1), GLint(v2), GLint(v3) ) -> None',
	argNames = ('location', 'v0', 'v1', 'v2', 'v3',),
)

glUniform1fv = platform.createExtensionFunction( 
	'glUniform1fv', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLint, constants.GLsizei, arrays.GLfloatArray,),
	doc = 'glUniform1fv( GLint(location), GLsizei(count), GLfloatArray(value) ) -> None',
	argNames = ('location', 'count', 'value',),
)

glUniform2fv = platform.createExtensionFunction( 
	'glUniform2fv', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLint, constants.GLsizei, arrays.GLfloatArray,),
	doc = 'glUniform2fv( GLint(location), GLsizei(count), GLfloatArray(value) ) -> None',
	argNames = ('location', 'count', 'value',),
)

glUniform3fv = platform.createExtensionFunction( 
	'glUniform3fv', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLint, constants.GLsizei, arrays.GLfloatArray,),
	doc = 'glUniform3fv( GLint(location), GLsizei(count), GLfloatArray(value) ) -> None',
	argNames = ('location', 'count', 'value',),
)

glUniform4fv = platform.createExtensionFunction( 
	'glUniform4fv', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLint, constants.GLsizei, arrays.GLfloatArray,),
	doc = 'glUniform4fv( GLint(location), GLsizei(count), GLfloatArray(value) ) -> None',
	argNames = ('location', 'count', 'value',),
)

glUniform1iv = platform.createExtensionFunction( 
	'glUniform1iv', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLint, constants.GLsizei, arrays.GLintArray,),
	doc = 'glUniform1iv( GLint(location), GLsizei(count), GLintArray(value) ) -> None',
	argNames = ('location', 'count', 'value',),
)

glUniform2iv = platform.createExtensionFunction( 
	'glUniform2iv', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLint, constants.GLsizei, arrays.GLintArray,),
	doc = 'glUniform2iv( GLint(location), GLsizei(count), GLintArray(value) ) -> None',
	argNames = ('location', 'count', 'value',),
)

glUniform3iv = platform.createExtensionFunction( 
	'glUniform3iv', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLint, constants.GLsizei, arrays.GLintArray,),
	doc = 'glUniform3iv( GLint(location), GLsizei(count), GLintArray(value) ) -> None',
	argNames = ('location', 'count', 'value',),
)

glUniform4iv = platform.createExtensionFunction( 
	'glUniform4iv', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLint, constants.GLsizei, arrays.GLintArray,),
	doc = 'glUniform4iv( GLint(location), GLsizei(count), GLintArray(value) ) -> None',
	argNames = ('location', 'count', 'value',),
)

glUniformMatrix2fv = platform.createExtensionFunction( 
	'glUniformMatrix2fv', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLint, constants.GLsizei, constants.GLboolean, arrays.GLfloatArray,),
	doc = 'glUniformMatrix2fv( GLint(location), GLsizei(count), GLboolean(transpose), GLfloatArray(value) ) -> None',
	argNames = ('location', 'count', 'transpose', 'value',),
)

glUniformMatrix3fv = platform.createExtensionFunction( 
	'glUniformMatrix3fv', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLint, constants.GLsizei, constants.GLboolean, arrays.GLfloatArray,),
	doc = 'glUniformMatrix3fv( GLint(location), GLsizei(count), GLboolean(transpose), GLfloatArray(value) ) -> None',
	argNames = ('location', 'count', 'transpose', 'value',),
)

glUniformMatrix4fv = platform.createExtensionFunction( 
	'glUniformMatrix4fv', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLint, constants.GLsizei, constants.GLboolean, arrays.GLfloatArray,),
	doc = 'glUniformMatrix4fv( GLint(location), GLsizei(count), GLboolean(transpose), GLfloatArray(value) ) -> None',
	argNames = ('location', 'count', 'transpose', 'value',),
)

glValidateProgram = platform.createExtensionFunction( 
	'glValidateProgram', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLuint,),
	doc = 'glValidateProgram( GLuint(program) ) -> None',
	argNames = ('program',),
)

glVertexAttrib1d = platform.createExtensionFunction( 
	'glVertexAttrib1d', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLuint, constants.GLdouble,),
	doc = 'glVertexAttrib1d( GLuint(index), GLdouble(x) ) -> None',
	argNames = ('index', 'x',),
)

glVertexAttrib1dv = platform.createExtensionFunction( 
	'glVertexAttrib1dv', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLuint, arrays.GLdoubleArray,),
	doc = 'glVertexAttrib1dv( GLuint(index), GLdoubleArray(v) ) -> None',
	argNames = ('index', 'v',),
)

glVertexAttrib1f = platform.createExtensionFunction( 
	'glVertexAttrib1f', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLuint, constants.GLfloat,),
	doc = 'glVertexAttrib1f( GLuint(index), GLfloat(x) ) -> None',
	argNames = ('index', 'x',),
)

glVertexAttrib1fv = platform.createExtensionFunction( 
	'glVertexAttrib1fv', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLuint, arrays.GLfloatArray,),
	doc = 'glVertexAttrib1fv( GLuint(index), GLfloatArray(v) ) -> None',
	argNames = ('index', 'v',),
)

glVertexAttrib1s = platform.createExtensionFunction( 
	'glVertexAttrib1s', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLuint, constants.GLshort,),
	doc = 'glVertexAttrib1s( GLuint(index), GLshort(x) ) -> None',
	argNames = ('index', 'x',),
)

glVertexAttrib1sv = platform.createExtensionFunction( 
	'glVertexAttrib1sv', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLuint, arrays.GLshortArray,),
	doc = 'glVertexAttrib1sv( GLuint(index), GLshortArray(v) ) -> None',
	argNames = ('index', 'v',),
)

glVertexAttrib2d = platform.createExtensionFunction( 
	'glVertexAttrib2d', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLuint, constants.GLdouble, constants.GLdouble,),
	doc = 'glVertexAttrib2d( GLuint(index), GLdouble(x), GLdouble(y) ) -> None',
	argNames = ('index', 'x', 'y',),
)

glVertexAttrib2dv = platform.createExtensionFunction( 
	'glVertexAttrib2dv', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLuint, arrays.GLdoubleArray,),
	doc = 'glVertexAttrib2dv( GLuint(index), GLdoubleArray(v) ) -> None',
	argNames = ('index', 'v',),
)

glVertexAttrib2f = platform.createExtensionFunction( 
	'glVertexAttrib2f', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLuint, constants.GLfloat, constants.GLfloat,),
	doc = 'glVertexAttrib2f( GLuint(index), GLfloat(x), GLfloat(y) ) -> None',
	argNames = ('index', 'x', 'y',),
)

glVertexAttrib2fv = platform.createExtensionFunction( 
	'glVertexAttrib2fv', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLuint, arrays.GLfloatArray,),
	doc = 'glVertexAttrib2fv( GLuint(index), GLfloatArray(v) ) -> None',
	argNames = ('index', 'v',),
)

glVertexAttrib2s = platform.createExtensionFunction( 
	'glVertexAttrib2s', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLuint, constants.GLshort, constants.GLshort,),
	doc = 'glVertexAttrib2s( GLuint(index), GLshort(x), GLshort(y) ) -> None',
	argNames = ('index', 'x', 'y',),
)

glVertexAttrib2sv = platform.createExtensionFunction( 
	'glVertexAttrib2sv', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLuint, arrays.GLshortArray,),
	doc = 'glVertexAttrib2sv( GLuint(index), GLshortArray(v) ) -> None',
	argNames = ('index', 'v',),
)

glVertexAttrib3d = platform.createExtensionFunction( 
	'glVertexAttrib3d', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLuint, constants.GLdouble, constants.GLdouble, constants.GLdouble,),
	doc = 'glVertexAttrib3d( GLuint(index), GLdouble(x), GLdouble(y), GLdouble(z) ) -> None',
	argNames = ('index', 'x', 'y', 'z',),
)

glVertexAttrib3dv = platform.createExtensionFunction( 
	'glVertexAttrib3dv', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLuint, arrays.GLdoubleArray,),
	doc = 'glVertexAttrib3dv( GLuint(index), GLdoubleArray(v) ) -> None',
	argNames = ('index', 'v',),
)

glVertexAttrib3f = platform.createExtensionFunction( 
	'glVertexAttrib3f', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLuint, constants.GLfloat, constants.GLfloat, constants.GLfloat,),
	doc = 'glVertexAttrib3f( GLuint(index), GLfloat(x), GLfloat(y), GLfloat(z) ) -> None',
	argNames = ('index', 'x', 'y', 'z',),
)

glVertexAttrib3fv = platform.createExtensionFunction( 
	'glVertexAttrib3fv', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLuint, arrays.GLfloatArray,),
	doc = 'glVertexAttrib3fv( GLuint(index), GLfloatArray(v) ) -> None',
	argNames = ('index', 'v',),
)

glVertexAttrib3s = platform.createExtensionFunction( 
	'glVertexAttrib3s', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLuint, constants.GLshort, constants.GLshort, constants.GLshort,),
	doc = 'glVertexAttrib3s( GLuint(index), GLshort(x), GLshort(y), GLshort(z) ) -> None',
	argNames = ('index', 'x', 'y', 'z',),
)

glVertexAttrib3sv = platform.createExtensionFunction( 
	'glVertexAttrib3sv', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLuint, arrays.GLshortArray,),
	doc = 'glVertexAttrib3sv( GLuint(index), GLshortArray(v) ) -> None',
	argNames = ('index', 'v',),
)

glVertexAttrib4Nbv = platform.createExtensionFunction( 
	'glVertexAttrib4Nbv', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLuint, arrays.GLbyteArray,),
	doc = 'glVertexAttrib4Nbv( GLuint(index), GLbyteArray(v) ) -> None',
	argNames = ('index', 'v',),
)

glVertexAttrib4Niv = platform.createExtensionFunction( 
	'glVertexAttrib4Niv', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLuint, arrays.GLintArray,),
	doc = 'glVertexAttrib4Niv( GLuint(index), GLintArray(v) ) -> None',
	argNames = ('index', 'v',),
)

glVertexAttrib4Nsv = platform.createExtensionFunction( 
	'glVertexAttrib4Nsv', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLuint, arrays.GLshortArray,),
	doc = 'glVertexAttrib4Nsv( GLuint(index), GLshortArray(v) ) -> None',
	argNames = ('index', 'v',),
)

glVertexAttrib4Nub = platform.createExtensionFunction( 
	'glVertexAttrib4Nub', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLuint, constants.GLubyte, constants.GLubyte, constants.GLubyte, constants.GLubyte,),
	doc = 'glVertexAttrib4Nub( GLuint(index), GLubyte(x), GLubyte(y), GLubyte(z), GLubyte(w) ) -> None',
	argNames = ('index', 'x', 'y', 'z', 'w',),
)

glVertexAttrib4Nubv = platform.createExtensionFunction( 
	'glVertexAttrib4Nubv', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLuint, arrays.GLubyteArray,),
	doc = 'glVertexAttrib4Nubv( GLuint(index), GLubyteArray(v) ) -> None',
	argNames = ('index', 'v',),
)

glVertexAttrib4Nuiv = platform.createExtensionFunction( 
	'glVertexAttrib4Nuiv', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLuint, arrays.GLuintArray,),
	doc = 'glVertexAttrib4Nuiv( GLuint(index), GLuintArray(v) ) -> None',
	argNames = ('index', 'v',),
)

glVertexAttrib4Nusv = platform.createExtensionFunction( 
	'glVertexAttrib4Nusv', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLuint, arrays.GLushortArray,),
	doc = 'glVertexAttrib4Nusv( GLuint(index), GLushortArray(v) ) -> None',
	argNames = ('index', 'v',),
)

glVertexAttrib4bv = platform.createExtensionFunction( 
	'glVertexAttrib4bv', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLuint, arrays.GLbyteArray,),
	doc = 'glVertexAttrib4bv( GLuint(index), GLbyteArray(v) ) -> None',
	argNames = ('index', 'v',),
)

glVertexAttrib4d = platform.createExtensionFunction( 
	'glVertexAttrib4d', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLuint, constants.GLdouble, constants.GLdouble, constants.GLdouble, constants.GLdouble,),
	doc = 'glVertexAttrib4d( GLuint(index), GLdouble(x), GLdouble(y), GLdouble(z), GLdouble(w) ) -> None',
	argNames = ('index', 'x', 'y', 'z', 'w',),
)

glVertexAttrib4dv = platform.createExtensionFunction( 
	'glVertexAttrib4dv', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLuint, arrays.GLdoubleArray,),
	doc = 'glVertexAttrib4dv( GLuint(index), GLdoubleArray(v) ) -> None',
	argNames = ('index', 'v',),
)

glVertexAttrib4f = platform.createExtensionFunction( 
	'glVertexAttrib4f', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLuint, constants.GLfloat, constants.GLfloat, constants.GLfloat, constants.GLfloat,),
	doc = 'glVertexAttrib4f( GLuint(index), GLfloat(x), GLfloat(y), GLfloat(z), GLfloat(w) ) -> None',
	argNames = ('index', 'x', 'y', 'z', 'w',),
)

glVertexAttrib4fv = platform.createExtensionFunction( 
	'glVertexAttrib4fv', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLuint, arrays.GLfloatArray,),
	doc = 'glVertexAttrib4fv( GLuint(index), GLfloatArray(v) ) -> None',
	argNames = ('index', 'v',),
)

glVertexAttrib4iv = platform.createExtensionFunction( 
	'glVertexAttrib4iv', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLuint, arrays.GLintArray,),
	doc = 'glVertexAttrib4iv( GLuint(index), GLintArray(v) ) -> None',
	argNames = ('index', 'v',),
)

glVertexAttrib4s = platform.createExtensionFunction( 
	'glVertexAttrib4s', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLuint, constants.GLshort, constants.GLshort, constants.GLshort, constants.GLshort,),
	doc = 'glVertexAttrib4s( GLuint(index), GLshort(x), GLshort(y), GLshort(z), GLshort(w) ) -> None',
	argNames = ('index', 'x', 'y', 'z', 'w',),
)

glVertexAttrib4sv = platform.createExtensionFunction( 
	'glVertexAttrib4sv', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLuint, arrays.GLshortArray,),
	doc = 'glVertexAttrib4sv( GLuint(index), GLshortArray(v) ) -> None',
	argNames = ('index', 'v',),
)

glVertexAttrib4ubv = platform.createExtensionFunction( 
	'glVertexAttrib4ubv', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLuint, arrays.GLubyteArray,),
	doc = 'glVertexAttrib4ubv( GLuint(index), GLubyteArray(v) ) -> None',
	argNames = ('index', 'v',),
)

glVertexAttrib4uiv = platform.createExtensionFunction( 
	'glVertexAttrib4uiv', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLuint, arrays.GLuintArray,),
	doc = 'glVertexAttrib4uiv( GLuint(index), GLuintArray(v) ) -> None',
	argNames = ('index', 'v',),
)

glVertexAttrib4usv = platform.createExtensionFunction( 
	'glVertexAttrib4usv', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLuint, arrays.GLushortArray,),
	doc = 'glVertexAttrib4usv( GLuint(index), GLushortArray(v) ) -> None',
	argNames = ('index', 'v',),
)

glVertexAttribPointer = platform.createExtensionFunction( 
	'glVertexAttribPointer', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLuint, constants.GLint, constants.GLenum, constants.GLboolean, constants.GLsizei, ctypes.c_void_p,),
	doc = 'glVertexAttribPointer( GLuint(index), GLint(size), GLenum(type), GLboolean(normalized), GLsizei(stride), c_void_p(pointer) ) -> None',
	argNames = ('index', 'size', 'type', 'normalized', 'stride', 'pointer',),
)




'''OpenGL extension VERSION.GL_2_0

This module customises the behaviour of the 
OpenGL.raw.GL.VERSION.GL_2_0 to provide a more 
Python-friendly API
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions, wrapper
from OpenGL.GL import glget
import ctypes
from OpenGL.raw.GL.VERSION.GL_2_0 import *
### END AUTOGENERATED SECTION
import OpenGL
from OpenGL.raw.GL.ARB.shader_objects import GL_OBJECT_COMPILE_STATUS_ARB as GL_OBJECT_COMPILE_STATUS
from OpenGL.raw.GL.ARB.shader_objects import GL_OBJECT_LINK_STATUS_ARB as GL_OBJECT_LINK_STATUS
from OpenGL.GL.ARB.shader_objects import glGetInfoLogARB as glGetInfoLog

from OpenGL import converters, error
GL_INFO_LOG_LENGTH = constant.Constant( 'GL_INFO_LOG_LENGTH', 0x8B84 )

glShaderSource = platform.createBaseFunction( 
	'glShaderSource', dll=platform.GL,
	resultType=None, 
	argTypes=(constants.GLhandle, constants.GLsizei, ctypes.POINTER(ctypes.c_char_p), arrays.GLintArray,),
	doc = 'glShaderSource( GLhandle(shaderObj), str( string) ) -> None',
	argNames = ('shaderObj', 'count', 'string', 'length',),
)
conv = converters.StringLengths( name='string' )
glShaderSource = wrapper.wrapper(
	glShaderSource
).setPyConverter(
	'count' # number of strings
).setPyConverter( 
	'length' # lengths of strings
).setPyConverter(
	'string', conv.stringArray
).setCResolver(
	'string', conv.stringArrayForC,
).setCConverter(
	'length', conv,
).setCConverter(
	'count', conv.totalCount,
)
del conv

for size in (1,2,3,4):
	for format,arrayType in (
		('f',arrays.GLfloatArray),
		('i',arrays.GLintArray),
	):
		name = 'glUniform%(size)s%(format)sv'%globals()
		globals()[name] = arrays.setInputArraySizeType(
			globals()[name],
			size,
			arrayType, 
			'value',
		)
		del format, arrayType
	del size

def _afterCheck( key ):
	"""Generate an error-checking function for compilation operations"""
	if key == GL_OBJECT_COMPILE_STATUS:
		getter = glGetShaderiv
	else:
		getter = glGetProgramiv
	def GLSLCheckError( 
		result,
		baseOperation=None,
		cArguments=None,
		*args
	):
		result = error.glCheckError( result, baseOperation, cArguments, *args )
		status = ctypes.c_int()
		getter( cArguments[0], key, ctypes.byref(status))
		status = status.value


		if not status:
			raise error.GLError( 
				result = result,
				baseOperation = baseOperation,
				cArguments = cArguments,
				description= glGetInfoLog( cArguments[0] )
			)
		return result
	return GLSLCheckError

if OpenGL.ERROR_CHECKING:
	glCompileShader.errcheck = _afterCheck( GL_OBJECT_COMPILE_STATUS )
if OpenGL.ERROR_CHECKING:
	glLinkProgram.errcheck = _afterCheck( GL_OBJECT_LINK_STATUS )
## Not sure why, but these give invalid operation :(
##if glValidateProgram and OpenGL.ERROR_CHECKING:
##	glValidateProgram.errcheck = _afterCheck( GL_OBJECT_VALIDATE_STATUS )

base_glGetShaderSource = glGetShaderSource
def glGetShaderSource( obj ):
	"""Retrieve the program/shader's source code as a Python string
	
	returns string which is '' if no source code
	"""
	length = int(glGetObjectParameteriv(obj, GL_OBJECT_SHADER_SOURCE_LENGTH))
	if length > 0:
		source = ctypes.create_string_buffer(length)
		base_glGetShaderSource(obj, length, None, source)
		return source.value.strip('\000') # null-termination
	return ''
glGetShaderSource.wrappedOperation = base_glGetShaderSource

base_glGetActiveUniform = glGetActiveUniform
def glGetActiveUniform(program, index):
	"""Retrieve the name, size and type of the uniform of the index in the program"""
	max_index = int(glGetObjectParameteriv( program, GL_OBJECT_ACTIVE_UNIFORMS ))
	length = int(glGetObjectParameteriv( program, GL_OBJECT_ACTIVE_UNIFORM_MAX_LENGTH))
	if index < max_index and index >= 0 and length > 0:
		name = ctypes.create_string_buffer(length)
		size = arrays.GLintArray.zeros( (1,))
		gl_type = arrays.GLuintArray.zeros( (1,))
		base_glGetActiveUniform(program, index, length, None, size, gl_type, name)
		return name.value, size[0], gl_type[0]
	raise IndexError, 'Index %s out of range 0 to %i' % (index, max_index - 1, )
glGetActiveUniform.wrappedOperation = base_glGetActiveUniform



'''OpenGL extension ARB.point_parameters

This module customises the behaviour of the 
OpenGL.raw.GL.ARB.point_parameters to provide a more 
Python-friendly API
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions, wrapper
from OpenGL.GL import glget
import ctypes
from OpenGL.raw.GL.ARB.point_parameters import *
### END AUTOGENERATED SECTION
from OpenGL.GL import glget
glget.addGLGetConstant( GL_POINT_SIZE_MIN_ARB, (1,) )
glget.addGLGetConstant( GL_POINT_SIZE_MAX_ARB , (1,) )
glget.addGLGetConstant( GL_POINT_FADE_THRESHOLD_SIZE_ARB, (1,) )
glget.addGLGetConstant( GL_POINT_DISTANCE_ATTENUATION_ARB, (3,) )

glPointParameterfvARB = arrays.setInputArraySizeType(
	glPointParameterfvARB,
	None, # XXX should be dependant on the pname field!
	arrays.GLfloatArray, 
	'params',
)



'''OpenGL extension EXT.index_array_formats

Overview (from the spec)
	
	This extends the number of packed vertex formats accepted by
	InterleavedArrays to include formats which specify color indexes
	rather than RGBA colors.

The official definition of this extension is available here:
	http://oss.sgi.com/projects/ogl-sample/registry/EXT/index_array_formats.txt

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_EXT_index_array_formats'
GL_IUI_V2F_EXT = constant.Constant( 'GL_IUI_V2F_EXT', 0x81AD )
GL_IUI_V3F_EXT = constant.Constant( 'GL_IUI_V3F_EXT', 0x81AE )
GL_IUI_N3F_V2F_EXT = constant.Constant( 'GL_IUI_N3F_V2F_EXT', 0x81AF )
GL_IUI_N3F_V3F_EXT = constant.Constant( 'GL_IUI_N3F_V3F_EXT', 0x81B0 )
GL_T2F_IUI_V2F_EXT = constant.Constant( 'GL_T2F_IUI_V2F_EXT', 0x81B1 )
GL_T2F_IUI_V3F_EXT = constant.Constant( 'GL_T2F_IUI_V3F_EXT', 0x81B2 )
GL_T2F_IUI_N3F_V2F_EXT = constant.Constant( 'GL_T2F_IUI_N3F_V2F_EXT', 0x81B3 )
GL_T2F_IUI_N3F_V3F_EXT = constant.Constant( 'GL_T2F_IUI_N3F_V3F_EXT', 0x81B4 )


def glInitIndexArrayFormatsEXT():
	'''Return boolean indicating whether this extension is available'''
	return extensions.hasGLExtension( EXTENSION_NAME )



'''OpenGL extension SGIX.texture_scale_bias

Overview (from the spec)
	
	This extension adds scale, bias, and clamp to [0, 1] operations to the 
	texture pipeline.
	These operations are applied to the filtered result of a texture lookup,
	before that result is used in the texture environment equations and
	before the texture color lookup table of SGI_texture_color_table, 
	if that extension exists.
	These operations are distinct from the scale, bias, and clamp operations
	that appear in the SGI_color_table extension, which are used to
	define a color lookup table.
	
	Scale and bias operations on texels can be used to better utilize the
	color resolution of a particular texture internal format (see EXT_texture).

The official definition of this extension is available here:
	http://oss.sgi.com/projects/ogl-sample/registry/SGIX/texture_scale_bias.txt

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_SGIX_texture_scale_bias'
GL_POST_TEXTURE_FILTER_BIAS_SGIX = constant.Constant( 'GL_POST_TEXTURE_FILTER_BIAS_SGIX', 0x8179 )
GL_POST_TEXTURE_FILTER_SCALE_SGIX = constant.Constant( 'GL_POST_TEXTURE_FILTER_SCALE_SGIX', 0x817A )
GL_POST_TEXTURE_FILTER_BIAS_RANGE_SGIX = constant.Constant( 'GL_POST_TEXTURE_FILTER_BIAS_RANGE_SGIX', 0x817B )
glget.addGLGetConstant( GL_POST_TEXTURE_FILTER_BIAS_RANGE_SGIX, (1,) )
GL_POST_TEXTURE_FILTER_SCALE_RANGE_SGIX = constant.Constant( 'GL_POST_TEXTURE_FILTER_SCALE_RANGE_SGIX', 0x817C )
glget.addGLGetConstant( GL_POST_TEXTURE_FILTER_SCALE_RANGE_SGIX, (1,) )


def glInitTextureScaleBiasSGIX():
	'''Return boolean indicating whether this extension is available'''
	return extensions.hasGLExtension( EXTENSION_NAME )



'''OpenGL extension ARB.texture_compression

This module customises the behaviour of the 
OpenGL.raw.GL.ARB.texture_compression to provide a more 
Python-friendly API
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions, wrapper
from OpenGL.GL import glget
import ctypes
from OpenGL.raw.GL.ARB.texture_compression import *
### END AUTOGENERATED SECTION
from OpenGL.GL import images

for dimensions in (1,2,3):
	for function in ('glCompressedTexImage%sDARB','glCompressedTexSubImage%sDARB'):
		name = function%(dimensions,)
		globals()[ name ] = images.compressedImageFunction(
			globals()[ name ]
		)
		del name, function
	del dimensions

if glGetCompressedTexImageARB:
	def glGetCompressedTexImageARB( target, level, img=None ):
		"""Retrieve a compressed texture image"""
		if img is None:
			length = glget.glGetTexLevelParameteriv(
				target, 0, 
				GL_TEXTURE_COMPRESSED_IMAGE_SIZE_ARB, 
			)
			img = arrays.ArrayDataType.zeros( (length,), constants.GL_UNSIGNED_BYTE )
		return glGetCompressedTexImageARB(target, 0, img);



'''OpenGL extension EXT.shared_texture_palette

Overview (from the spec)
	
	EXT_shared_texture_palette defines a shared texture palette which may be
	used in place of the texture object palettes provided by
	EXT_paletted_texture. This is useful for rapidly changing a palette
	common to many textures, rather than having to reload the new palette
	for each texture. The extension acts as a switch, causing all lookups
	that would normally be done on the texture's palette to instead use the
	shared palette.

The official definition of this extension is available here:
	http://oss.sgi.com/projects/ogl-sample/registry/EXT/shared_texture_palette.txt

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_EXT_shared_texture_palette'
GL_SHARED_TEXTURE_PALETTE_EXT = constant.Constant( 'GL_SHARED_TEXTURE_PALETTE_EXT', 0x81FB )


def glInitSharedTexturePaletteEXT():
	'''Return boolean indicating whether this extension is available'''
	return extensions.hasGLExtension( EXTENSION_NAME )



"""Storage of per-context values of various types

Because OpenGL needs persistent references to the
objects we're constructing to shadow Python objects,
we have to store references to the objects somewhere

For any given Python GUI library, we can use a weakref
to the library's representation of the GL context to 
call the cleanup function.  That means some per-GUI 
library code in OpenGL (or the library), but it gives 
us very natural operations within OpenGL.
"""
from OpenGL import platform
storedPointers = {
	# map from contextID: { constant: value }
}
storedWeakPointers = {
	# map from contextID: WeakValueDictionary({ constant: value })
}
STORAGES = [ storedPointers, storedWeakPointers ]

def getContext( context = None ):
	"""Get the context (if passed, just return)
	
	context -- the context ID, if None, the current context
	"""
	if context is None:
		context = platform.GetCurrentContext()
		if context == 0:
			from OpenGL import error
			raise error.Error(
				"""Attempt to retrieve context when no valid context"""
			)
	return context
def setValue( constant, value, context=None, weak=False ):
	"""Set a stored value for the given context
	
	constant -- Normally a GL constant value, but can be any hashable value 
	value -- the value to be stored.  If weak is true must be 
		weak-reference-able.  If None, then the value will be deleted from 
		the storage 
	context -- the context identifier for which we're storing the value
	weak -- if true, value will be stored with a weakref
		Note: you should always pass the same value for "weak" for a given 
		constant, otherwise you will create two storages for the constant.
	"""
	if getattr( value, '_no_cache_', False ):
		return 
	context = getContext( context )
	if weak:
		storage = storedWeakPointers
		cls = weakref.WeakValueDictionary
	else:
		storage = storedPointers
		cls = dict
	current = storage.get( context )
	if current is None:
		storage[context] = current = cls()
	previous = current.get( constant )
	if value is None:
		try:
			del current[ constant ]
		except (KeyError,TypeError,ValueError), err:
			pass 
	else:
		# XXX potential for failure here if a non-weakref-able objects
		# is being stored with weak == True
		current[ constant ] = value 
##	print 'previous', previous, value, constant
	return previous
def delValue( constant, context=None ):
	"""Delete the specified value for the given context
	
	constant -- Normally a GL constant value, but can be any hashable value 
	context -- the context identifier for which we're storing the value
	"""
	context = getContext( context )
	found = False
	for storage in STORAGES:
		contextStorage = storage.get( context  )
		if contextStorage:
			try:
				del contextStorage[ constant ]
				found = True
			except KeyError, err:
				pass
	return found

def getValue( constant, context = None ):
	"""Get a stored value for the given constant
	
	constant -- unique ID for the type of data being retrieved
	context -- the context ID, if None, the current context
	"""
	context = getContext( context )
	for storage in STORAGES:
		contextStorage = storage.get( context  )
		if contextStorage:
			value =  contextStorage.get( constant )
			if value is not None:
				return value
	return None

def cleanupContext( context=None ):
	"""Cleanup all held pointer objects for the given context
	
	Warning: this is dangerous, as if you call it before a context 
	is destroyed you may release memory held by the context and cause
	a protection fault when the GL goes to render the scene!
	
	Normally you will want to get the context ID explicitly and then 
	register cleanupContext as a weakref callback to your GUI library 
	Context object with the (now invalid) context ID as parameter.
	"""
	if context is None:
		context = platform.GetCurrentContext()
	for storage in STORAGES:
		try:
			del storedPointers[ context ]
		except KeyError, err:
			return False
		else:
			return True



'''OpenGL extension EXT.stencil_two_side

Overview (from the spec)
	
	This extension provides two-sided stencil testing where the
	stencil-related state (stencil operations, reference value, compare
	mask, and write mask) may be different for front- and back-facing
	polygons.  Two-sided stencil testing may improve the performance
	of stenciled shadow volume and Constructive Solid Geometry (CSG)
	rendering algorithms.

The official definition of this extension is available here:
	http://oss.sgi.com/projects/ogl-sample/registry/EXT/stencil_two_side.txt

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_EXT_stencil_two_side'
GL_STENCIL_TEST_TWO_SIDE_EXT = constant.Constant( 'GL_STENCIL_TEST_TWO_SIDE_EXT', 0x8910 )
glget.addGLGetConstant( GL_STENCIL_TEST_TWO_SIDE_EXT, (1,) )
GL_ACTIVE_STENCIL_FACE_EXT = constant.Constant( 'GL_ACTIVE_STENCIL_FACE_EXT', 0x8911 )
glget.addGLGetConstant( GL_ACTIVE_STENCIL_FACE_EXT, (1,) )
glActiveStencilFaceEXT = platform.createExtensionFunction( 
	'glActiveStencilFaceEXT', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum,),
	doc = 'glActiveStencilFaceEXT( GLenum(face) ) -> None',
	argNames = ('face',),
)


def glInitStencilTwoSideEXT():
	'''Return boolean indicating whether this extension is available'''
	return extensions.hasGLExtension( EXTENSION_NAME )
